// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.List;
import java.util.LinkedList;
import java.util.Map;
import java.util.HashMap;
import java.util.Collection;
import java.util.Iterator;
import java.io.Serializable;

/**
 * This class holds the statistics for a single 
 * exercise on some course
 */
public class ExerciseStatistics implements Serializable {

    private int nroExercise;
    private int maxPoints;
    private int minPoints;
    private int totalPoints;
    private int totalAnswers;
    private int nroStudents;
    private float maxAve;
    //private List statistics;
    private HashMap studentPoints;


    public ExerciseStatistics() {
	//statistics = new LinkedList();
	studentPoints = new HashMap(100);
	maxPoints = 0;
	minPoints = Integer.MAX_VALUE;
	totalPoints = totalAnswers = 0;
	nroStudents = 0;
	maxAve = -1;
    }

    public void addStatistics(String nro, int p) {
	//statistics.add(new Stat(nro, p));
	Integer pt = (Integer)studentPoints.get(nro);
	if(pt == null || pt.intValue() < p) {
	    studentPoints.put(nro, new Integer(p));
	    if(pt == null) 
		nroStudents++;
	}
	totalPoints += p;
	totalAnswers++;
	if(p > maxPoints) {
	    maxPoints = p;
	} 
	if( p < minPoints) {
	    minPoints = p;
	}
    }

    public int getAmountOfStudents() {
	return nroStudents;
    }
    
    /** 
     * The average of students' best answers to this exercise. Will also
     * calculate the average of total answers as well as the amount of students
     * who have returned this exercise.
     */
      public float getAverageOfMaximums() {
	  if(maxAve < 0) {
	      Iterator i = studentPoints.values().iterator();
	      float val = 0;
	      while(i.hasNext()) {
		  val += ((Integer)i.next()).floatValue();
	      }
	      maxAve = val / (float)studentPoints.size();
	  }
	  return maxAve;
      }

    /** Average of all answers to this exercise */
    public float getExerciseAverage() {
	return ((float)totalPoints / (float)totalAnswers);
    }

    /** The number of answers to this exercise */
    public int getAmountOfAnswers() {
	//return statistics.size();
	return totalAnswers;
    }

    public int getMax() {
	return maxPoints;
    }

    public int getMin() {
	if(minPoints == Integer.MAX_VALUE) return 0;
	return minPoints;
    }

    public int getExerciseNumber() {
	return nroExercise;
    }

    public void setExerciseNumber(int n) {
	nroExercise = n;
    }

    public Map getStudentPoints() {
	return studentPoints;
    }

    private class Stat {
	String opNro;
	int points;

	Stat(String op, int p) {
	    opNro = op;
	    points = p;
	}
    }



}
