// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.io.Serializable;
import java.util.HashMap;

/**
 * Organization. Business logic class that provides information about the organizing entity for a course.
 * Can also be used to customize the TRAKLA2-environment for the selected organization.
 *
 * @author cefallen
 *
 */
public class Organization implements Serializable {

    /**
     * Id for serializing
     */
    private static final long serialVersionUID = 1L;

    /** 
     * Organization id, must be unique 
     */
    private int id;

    /** 
     * Organization acronyms in different locales
     */
    private HashMap acronyms;

    /** 
     * Organization names in different locales
     */
    private HashMap names;

    /** 
     * Organization descriptions in different locales
     */
    private HashMap descriptions;

    /** 
     * Links to organization home pages in different locales
     */
    private HashMap links;

    /** Shibboleth login url */
    private String shibId;

    public Organization(int id) {
        this.id = id;
        acronyms = new HashMap();
        names = new HashMap();
        descriptions = new HashMap();
        links = new HashMap();
    }

    public int getId() {
        return id;
    }

    public void addLocalizedAcronym(String lang, String localizedAcronym) {
        if(lang != null && localizedAcronym != null)
            acronyms.put(lang, localizedAcronym);
    }

    public String getLocalizedAcronym(String lang) {
        String temp = (String)acronyms.get(lang);
        if(temp != null)
            return temp;
        return "";
    }

    public void addLocalizedName(String lang, String localizedName) {
        if(lang != null && localizedName != null)
            names.put(lang, localizedName);
    }

    public String getLocalizedName(String lang) {
        String temp = (String)names.get(lang);
        if(temp != null)
            return temp;
        return "";
    }

    public void addLocalizedDescription(String lang, String localizedDescription) {
        if(lang != null && localizedDescription != null)
            descriptions.put(lang, localizedDescription);
    }

    public String getLocalizedDescription(String lang) {
        String temp = (String)descriptions.get(lang);
        if(temp != null)
            return temp;
        return "";
    }

    public void addLocalizedLink(String lang, String localizedLink) {
        if(lang != null && localizedLink != null)
            links.put(lang, localizedLink);
    }

    public String getLocalizedLink(String lang) {
        String temp = (String)links.get(lang);
        if(temp != null)
            return temp;
        return "";
    }

    public void setShibId(String shibId) {
        this.shibId = shibId;
    }

    /**
     * Returns shibboleth login url or null if it's not set.
     */
    public String getShibId() {
        if (shibId == null || shibId.length() < 1) {
            return null;
        } else {
            return shibId;
        }
    }
}
