// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.List;
import java.util.LinkedList;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Collection;
import java.util.Set;
import java.io.Serializable;

/**
 * This class holds the statistics for a single
 * round on some course. Holds data only for those
 * students that have actually returned something for this round.
 */
public class RoundStatistics  implements Serializable {

    private int nroRound;
    private int nroAnswers;
    private int roundMax;
    private List exercises;
    private HashMap studentPoints;
    private Integer[] pointAmounts;

    public RoundStatistics() {
	exercises = new LinkedList();
	studentPoints = new HashMap();
	nroAnswers = -1;
	roundMax = 0;
	pointAmounts = null;

    }

    public int getRoundNumber() {
	return nroRound;
    }

    public void setRoundNumber(int r) {
	nroRound = r;
    }

    public int getAmountOfAnswers() {
	if(nroAnswers < 0) {
	    int a = 0;
	    for(int i = 0; i < exercises.size(); i++) {
		a += ((ExerciseStatistics)exercises.get(i)).getAmountOfAnswers();
	    }
	    nroAnswers = a;
	}
	return nroAnswers;
    }

    public int getAmountOfStudents() {
	return studentPoints.size();
    }

    public void addExercise(ExerciseStatistics es) {
	//System.out.println("Adding statistics for exercise " + getRoundNumber()+"."+ es.getExerciseNumber());
	exercises.add(es);
    }
    
    public ExerciseStatistics getExercise(int nro) {
	return (ExerciseStatistics)exercises.get(nro);
    }

    public List getExercises() {
	return exercises;
    }

    private HashMap fetchStudentPoints() {
	Iterator i = null;
	if(studentPoints.isEmpty()) {
	    i = exercises.iterator();
	    while(i.hasNext()) {
		ExerciseStatistics es = (ExerciseStatistics)i.next();
		roundMax += es.getMax();
		Map m = es.getStudentPoints();
		Iterator j = m.keySet().iterator();
		while(j.hasNext()) {
		    String opNro = (String)j.next();
		    Integer points = (Integer)m.get(opNro);
		    Integer oldPoints = (Integer)studentPoints.get(opNro);
		    if(oldPoints == null)
			studentPoints.put(opNro,points);
		    else
			studentPoints.put(opNro,new Integer(points.intValue()+oldPoints.intValue()));
		}
	    }
	}
	return studentPoints;
    }

    public Integer[] getRoundPointAmounts() {
	if(pointAmounts == null) {
	    HashMap map = fetchStudentPoints();
	    Iterator i = map.values().iterator();
	    int[] tmp = new int[roundMax+1];
	    while(i.hasNext()) {
		int val = ((Integer)i.next()).intValue();
		tmp[val]++;
	    }
	    pointAmounts = new Integer[tmp.length];
	    for(int cnt = 0; cnt < pointAmounts.length;cnt++) {
		pointAmounts[cnt] = new Integer(tmp[cnt]);
	    }
	}
	return pointAmounts;
		    
    }

    public double getRoundAverage() {
	HashMap map = fetchStudentPoints();
	Iterator i = map.values().iterator();
	double total = 0;
	while(i.hasNext()) 
	    total += ((Integer)i.next()).doubleValue();
	return total/fetchStudentPoints().size();
    }

    public int getRoundMax() {
	return roundMax;
    }

    public Map getStudentPoints() {
	return fetchStudentPoints();
    }

}
