// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.List;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.util.Collection;
import java.util.Set;
import java.io.Serializable;

/**
 * This class holds the statistics for a single course.
 * The statistics are created by the DataStorage on demand.
 */
public class Statistics implements Serializable {

    /** Number of students registered to this course */
    private int nroStudents;
    /** Number of students that have been unregistered */
    private int unregistered;
    /** Number of answers returned for this course */
    private int nroAnswers;
    private int max;
    private List rounds;
    private HashMap studentPoints;

    private int id;

    public Statistics(int i) {
	nroAnswers = -1;
	rounds = new LinkedList();
	studentPoints = new HashMap();
	id = i;
	max = 0;
    }

    public int getId() {
	return id;
    }

    public int getNroStudents() {
	return nroStudents;
    }

    public void setNroStudents(int n) {
	nroStudents = n;
    }

    public int getNroUnregistered() {
	return unregistered;
    }

    public void setNroUnregistered(int n) {
	unregistered = n;
    }

    public int getNroAnswers() {
	if(nroAnswers < 0) {
	    int a = 0;
	    for(int i = 0; i < rounds.size(); i++) {
		a += ((RoundStatistics)rounds.get(i)).getAmountOfAnswers();
	    }
	    nroAnswers = a;
	}
	return nroAnswers;
    }

    public void setRoundStatistics(List l) {
	rounds = l;
    }

    public void addRound(RoundStatistics rs) {
	rounds.add(rs);
    }

    public RoundStatistics getRoundStatistics(int i) {
	return (RoundStatistics)rounds.get(i);
    }

    public List getRounds() {
	return rounds;
    }

    private HashMap fetchStudentPoints() {
	Iterator i = null;
	if(studentPoints.isEmpty()) {
	    i = rounds.iterator();
	    while(i.hasNext()) {
		RoundStatistics rs = (RoundStatistics)i.next();
		Map m = rs.getStudentPoints();
		max += rs.getRoundMax();
		Iterator j = m.keySet().iterator();
		while(j.hasNext()) {
		    String opNro = (String)j.next();
		    Integer points = (Integer)m.get(opNro);
		    Integer oldPoints = (Integer)studentPoints.get(opNro);
		    if(oldPoints == null)
			studentPoints.put(opNro,points);
		    else
			studentPoints.put(opNro,new Integer(points.intValue()+oldPoints.intValue()));
		}
	    }
	}
	return studentPoints;
    }

    public Integer[] getPointAmounts() {
	HashMap map = fetchStudentPoints();
	Iterator i = map.values().iterator();
	int[] tmp = new int[max+1];
	while(i.hasNext()) {
	    int val = ((Integer)i.next()).intValue();
	    tmp[val]++;
	}
	Integer[] pts = new Integer[tmp.length];
	for(int cnt = 0; cnt < pts.length;cnt++) {
	    pts[cnt] = new Integer(tmp[cnt]);
	}
	return pts;

    }

}
