// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.components;

import org.apache.tapestry.BaseComponent;

import java.util.Calendar;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;


import cexer.*;
import cexer.util.FingerprintHelper;
import cexer.util.Logging;

public abstract class Applet extends BaseComponent {

    public abstract Course getCourse();

    public abstract Round getRound();

    public abstract Exercise getExercise();

    public abstract Student getStudent();

    public abstract String getUserid();

    public abstract StudentSession getStudentSession();

    //public void setFontSize(int value) { fontSize = value; }
    //public int getFontSize() { return fontSize; }

    public String getExerciseNumber() {
        int round = getRound().getRoundNumber();
        int exercise = getExercise().getNumber();
        return round + "." + exercise;
    }

    /**
     * Gets the additional applet parameters for the current course (or for the
     * dummy course if the user is testing the exercise from the monitor interface),
     * round and exercise.
     * @return
     */
    public List getAdditionalAppletParameters() {
        Hashtable parameters = new Hashtable();

        // course parameters
        Iterator iter = getCourse().getAdditionalAppletParameters().iterator();
        if(getStudentSession().getUsingMonitorUI()) {
            DataStorage ds = DataStorage.getInstance();
            Course tempCourse = ds.getCourse(ds.DUMMY_COURSE);
            iter = tempCourse.getAdditionalAppletParameters().iterator();
        }

        while(iter.hasNext()) {
            AppletParameter param = (AppletParameter)iter.next();
            parameters.put(param.getName(), param);
        }

        //round parameters
        iter = getRound().getAdditionalAppletParameters().iterator();
        while(iter.hasNext()) {
            AppletParameter param = (AppletParameter)iter.next();
            parameters.put(param.getName(), param);
        }

        // exercise specific parameters
        iter = getExercise().getAppletParameters().iterator();
        while(iter.hasNext()) {
            AppletParameter param = (AppletParameter)iter.next();
            parameters.put(param.getName(), param);
        }

        return new LinkedList(parameters.values());
    }
    
    public String getPort() {
        String port = System.getProperty("rmi.port");
        if(port == null)
            port = "-1";
        return port;
    }
    
    public String getHost() {
        String host = System.getProperty("rmi.server");
        if(host == null)
            host = "";
        return host;        
    }

    /**
     * Gets the course id.
     * @return
     */
    public int getCourseId() {
        if(getStudentSession().getUsingMonitorUI()) {
            return DataStorage.DUMMY_COURSE;
        } else {
            return getCourse().getId();
        }
    }


    // Here because Exercise doesn't know it's Round -> doesn't
    // know it's DL
    public int getMaxPoints() {
        // Always return the full points. The submission part handles
        // the late submissions. -vkaravir 23.11.2004
        //	if (isBeforeDeadLine())
        return getExercise().getMax();
        //else
        //return 0;//(int)Math.ceil(getExercise().getMax()/2.0);
    }

    public boolean isBeforeDeadLine() {
        boolean b = false;

        if(getStudent().hasPersonalDeadLine(getRound().getId())) {
            b = getStudent().getDeadLine(getRound().getId()).after(Calendar.getInstance().getTime());
        }

        if(b == false) {
            return getRound().getDeadLine().after(Calendar.getInstance().getTime());
        }

        return b;
    }

    public String getFingerprint() {
        Exercise e = getExercise();
        ExerciseTemplate et = e.getExerciseTemplate();
        List params = e.getAppletParameters();
        String exClass = null;

        for (int i=0; i<params.size(); i++) {
            AppletParameter param = (AppletParameter)params.get(i);
            if (param.getName().equals("ex")) {
                exClass = param.getValue();
                break;
            }
        }

        if (exClass == null) {
            params = getRound().getAdditionalAppletParameters();
            for (int i=0; i<params.size(); i++) {
                AppletParameter param = (AppletParameter)params.get(i);
                if (param.getName().equals("ex")) {
                    exClass = param.getValue();
                    break;
                }
            }
        }

        if (exClass == null) {
            params = getCourse().getAdditionalAppletParameters();
            for (int i=0; i<params.size(); i++) {
                AppletParameter param = (AppletParameter)params.get(i);
                if (param.getName().equals("ex")) {
                    exClass = param.getValue();
                    break;
                }
            }
        }

        String fingerprint = FingerprintHelper.getFingerprint("" + getCourse().getId(),
                            getRound().getRoundNumber(),
                            e.getNumber(), getStudent().getId(),
                            getMaxPoints(), exClass);

        try {
            return FingerprintHelper.calculateMD5(fingerprint);
        } catch (NoSuchAlgorithmException exp) {
            exp.printStackTrace();
            Logging.error(this, "Exception calculating checksum", exp);
            return fingerprint;
        }
    }

}
