// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * 
 */
package cexer.components;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;

import cexer.DataStorage;
import cexer.MonitorSession;
import cexer.pages.MonitorModifyDescriptionPage;
import cexer.util.Trakla2SessionListener;

/**
 * Border class for the monitor user interface of the TRAKLA2 application.
 * 
 * @author cefallen
 *
 */
public abstract class MonitorBorder extends BaseComponent {

    /**
     * Gets the monitor session state object
     * 
     * @return monitor session
     */
    public abstract MonitorSession getMonitorSession();
    
    public String getRefresh() {
        // Make sure the properties are loaded
        DataStorage.getInstance();
        return "<meta HTTP-EQUIV=\"refresh\" CONTENT=\"0;URL=http://" + System.getProperty("http.server") + "/monitor\"/>";
    }
    
    /**
     * Gets the logout-link.
     * 
     * @param cycle
     * @return link to logout-procedure
     */
    public ILink logout(IRequestCycle cycle) {
        // do your own cleanup here
        cycle.forgetPage(cycle.getPage().getPageName());
        return getRestartService().getLink(false, null);
    }
    
    /**
     * Gets the tapestry restart-service, used for clearing the session data at logout.
     * 
     * @return restart service
     */
    public abstract IEngineService getRestartService();
    
    /**
     * Asks the framework if the Visit-object exists, e.g. if the HttpSession has been created
     * or if the current session is stateless.
     * @return TRUE if the StudentSession-object is created, otherwise FALSE
     */
    public abstract boolean getSessionState();
    
    public boolean isLogonTrue() {
        if(!getSessionState()) 
            return false;
        MonitorSession session = getMonitorSession();
        if (!session.getIsLoggedIn())
            return false;
        if (session.getUserId() == null)
            return false;
        if (session.getMonitor() == null)
            return false;
        if (!session.isInstructor() && !session.isAdmin())
            return false;
        return true;
    }

    public boolean isLogonOk() {
        String pageName = getPage().getPageName();
        // XXX This ain't nice, I know! But it works, though..
        boolean answer = isLogonTrue() 
        || pageName.equals("MonitorLogin")
        || pageName.equals("ForgotPassword");     
        return (answer);
    }
    
    /**
     * Activates the All Exercises page
     * 
     * @param cycle
     */
    public void allExercisesPage(IRequestCycle cycle) {
        if(getPage().getPageName().equals("MonitorModifyDescription"))
            ((MonitorModifyDescriptionPage)getPage()).externalReleaseLock();
        MonitorSession session = getMonitorSession();
        DataStorage ds = DataStorage.getInstance();
        session.setRound(ds.getAllExercises());
        session.setCourse(ds.getCourse(DataStorage.DUMMY_COURSE));
        cycle.activate("MonitorAllExercises");
    }
    
    /**
     * Gets the number of logged in users.
     * 
     * @return number of users
     */
    public int getNumberOfUsersOnline() {
        return Trakla2SessionListener.getNumberOfSessions();
    }
    
    /**
     * Gets a string containing the server name and the time of the server clock
     * 
     * @return status string
     */
    public String getStatusString() {
        DateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
        return "server: " + System.getProperty("http.server") + " - server time: " + df.format(new Date());
    }
    
    /**
     * Activates the login page of the application.
     * 
     * @param cycle
     */
    public void loginPage(IRequestCycle cycle) {
        String nextPage = "Login";
        if (cycle.getInfrastructure().getRequest().getActivationPath().equals("/monitor")) {
            nextPage = "MonitorLogin";
        }
        cycle.forgetPage(cycle.getPage().getPageName());
        cycle.activate(nextPage);
        return;
    }
}
