// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.components;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
//v4 import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.util.StringSplitter;

import java.util.List;
import java.util.Calendar;

import cexer.StudentSession;
import cexer.Visit;
import cexer.Round;
import cexer.Course;
import cexer.Exercise;
import cexer.Student;

/** Functionality for component RoundTable. */
public abstract class RoundTable extends BaseComponent {
    public static final String DEFAULT_TYPE = "student";
    private Round round;
    private Exercise exercise;
    private Course course;

    public Course getCourse() { return course; }
    public void setCourse(Course c) { course = c; }
    public Round getRound() { return round; }
    public void setRound(Round value) { round = value; }
    public Exercise getExercise() { return exercise; }
    public void setExercise(Exercise value) { exercise = value; }
    
    public void removeRoundAction(IRequestCycle cycle) {
	Course c = (Course)getBinding("course").getObject();
	Round r = getRound();
	c.removeRound(r);
	cycle.activate("AdminAddCourse");	
    }


    public Object[] getParameters() {
	Course c = (Course)getBinding("course").getObject();
	Round r = getRound();
	Exercise e = getExercise();
	//v3: List rounds = course.getRoundList();
    List rounds = c.getRoundList();
	List exercises = r.getExerciseList();
	Object[] params = new Object[3];
    StudentSession session = getStudentSession();
	//Visit visit = (Visit)getPage().getVisit();
	List courses = session.getStudent().getCourseList();
	int ci = courses.indexOf(c);
	int ri = rounds.indexOf(r);
	int ei = exercises.indexOf(e);
	params[0] = new Integer(ci);
	params[1] = new Integer(ri);
	params[2] = new Integer(ei);
	return params;
    }

    public void exerciseSelectAction(IRequestCycle cycle) {
	// This looks clumsy, but it is done to make sure that 
	// reload button works in the browsers.
  	//Visit visit = (Visit)getPage().getVisit();
    StudentSession session = getStudentSession();
	Object[] params = cycle.getListenerParameters();
	List courses = session.getStudent().getCourseList();
  	Course course = (Course)courses.get(((Integer)params[0]).intValue());
	List rounds = course.getRoundList();
  	Round round = (Round)rounds.get(((Integer)params[1]).intValue());
	List exercises = round.getExerciseList();
  	Exercise exercise = (Exercise)exercises.get(((Integer)params[2]).intValue());
  	session.setCourse(course);
  	session.setRound(round);
  	session.setExercise(exercise);
  	cycle.activate("Exercise");
    }

    /** Returns true, if the attribute type was given as admin. */
    public boolean isAdminType() {
        //v3: String type = getBinding("type").getString();
        String type = getType();
	return type.equals("admin");
    }

    //v4: 
    public abstract String getType();

    /*
     * Tapesty 4 application state object. Fetches the injected student-session
     * object (see Login.page)
     */

    public abstract StudentSession getStudentSession();

    
    /** Returns true, if the attribute type was given as student. */
    public boolean isStudentType() {
	String type = (String)getBinding("type").getObject();
	return type.equals("student");
    }

    /** Return the default value for the type of this component. */
    public String getDefaultType() {
	return DEFAULT_TYPE;
    }

    public String getExerciseStatus() {
	if (getPoints() == 0 && getSubmissions() == 0)
	    return "undoneExercise";
	else if (getPoints() == getExercise().getMax())
	    return "doneExercise";
	else
	    return "startedExercise";
    }
    
    public int getPoints() {
        StudentSession session = getStudentSession();
        Course course = (Course)getBinding("course").getObject();
        return session.getStudent().getResult(course, getRound(), getExercise());
    }

    /** Returns the localized name for the Exercise. */
    public String getExerciseName() {
	return getExercise().getLocalizedName(getPage().getLocale().getLanguage());
    }

    /** Returns the localized name for the Round. */
    public String getRoundName() {
	return getRound().getLocalizedName(getPage().getLocale().getLanguage());
    }

    /** Returns the localized description of the Round. */
    public String getRoundDescription() {
	return getRound().getLocalizedDescription(getPage().getLocale().getLanguage());
    }	

    public int getSubmissions() {
        StudentSession session = getStudentSession();
        Course course = (Course)getBinding("course").getObject();
        return session.getStudent().getSubmissions(course, getRound(), getExercise());	
    }

    /** 
     * Returns the file name of the icons used for exercises that are either
     * open or the deadline has passed. */
    public String getTimeIconFileName() {
	Calendar now = Calendar.getInstance();
	Calendar deadLine = Calendar.getInstance(getPage().getLocale());
	deadLine.setTime(getRound().getDeadLine());
	if (getHasPersonalDeadline())
        deadLine.setTime(getStudentSession().getStudent().getDeadLine(getRound().getId()));
        //deadLine.setTime(((Visit)getPage().getVisit()).getStudent().getDeadLine(getRound().getId()));
	if (now.after(deadLine))
	    return "images/red_ball.png";
	else
	    return "images/green_ball.png";
    }

    public int getStudentRoundPoints() {
        StudentSession session = getStudentSession();
        Round r = getRound();
        Course c = getCourse();
        Student s = session.getStudent();
        return s.getRoundResult(c, r.getRoundNumber());
    }

    public String getPointClass() {
        if (getStudentRoundPoints() < getRound().getMin())
            return "warning";
        else if (getStudentRoundPoints() == getRound().getMax())
            return "doneExercise";
        else
            return "startedExercise";
    }

    public boolean getHasPersonalDeadline() {
        StudentSession session = getStudentSession();
        return session.getStudent().hasPersonalDeadLine(getRound().getId());
    }

    public String getPersonalDeadline() {
        StudentSession session = getStudentSession();
        return session.getStudent().getFormattedDeadLine(getRound().getId());
    }

    public boolean isLimitedSubmissions() {
	int minSubs = getRound().getMinimumSubmissions();
	if (minSubs > 100)
	    return false;
	else
	    return true;
    }
}
