// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * 
 */
package cexer.pages;

import cexer.Course;
import cexer.DataStorage;
import cexer.StudentSession;
import java.util.Collection;
import java.util.List;
import cexer.Student;
import cexer.util.CourseSelectionModel;
import cexer.util.Logging;

import org.apache.tapestry.IRequestCycle;

/**
 * @author cefallen
 *
 */
public abstract class FeedbackPage extends SecurePage {

    /**
     * Tapesty 4 application state object. Fetches the injected student-session
     * object (see Login.page)
     */

    public abstract StudentSession getStudentSession();
    
    /**
     * Asks the framework if the StudentSession-object exists, e.g. if the HttpSession has been created
     * or if the current session is stateless.
     * @return TRUE if the StudentSession-object is created, otherwise FALSE
     */
    public abstract boolean getSessionState();
    
    
    /**
     * @return A CourseSelectionModel containing the courses that the user has enrolled to, or all courses
     * if the user hasn't logged in.
     */
    public Collection getCourseList() {
        DataStorage ds = DataStorage.getInstance();
        if(getSessionState() && getStudentSession().getStudent() != null) {
            Student s = getStudentSession().getStudent();
            return s.getCourseList();
        } else {
            return ds.getCourseSelectionModel().getList();
        }
    }
    
    /* public abstract void setFeedbackSent(boolean value);
    public abstract boolean getFeedbackSent(); */
    
    public void confirmFeedbackSent(IRequestCycle cycle) {
        //setFeedbackSent(true);
        cycle.activate(this);
    }
    
    public boolean getFeedbackSent() {        
        String sent = getRequestCycle().getRequestContext().getParameter("sent");
        //System.out.println("feedbackSent= " + sent);
        if(sent != null && sent.equals("true"))
            return true;
        return false;
    }
}