// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.valid.ValidationDelegate;
import cexer.Student;
import cexer.StudentSession;
import cexer.DataStorage;
import cexer.util.Logging;
import cexer.util.PasswordService;

//for email sending
import java.util.Properties;
import javax.mail.*;
import javax.mail.internet.*;


public abstract class ForgotPasswordPage extends SecurePage {

    /*
     * Tapesty 4 application state object. Fetches the injected student-session
     * object
     */
    public abstract StudentSession getStudentSession();

    public abstract void setWarning(boolean state);

    public abstract void setWarningText(String warningText);

    public abstract String getUserId();

    public abstract String getEmail();

    public void formSubmit(IRequestCycle cycle) {
        setWarning(false);

        ValidationDelegate delegate = (ValidationDelegate)getBeans().getBean("delegate");
        if (delegate.getHasErrors()) {
            cycle.activate(this);
            return;
        }

        DataStorage ds = DataStorage.getInstance();
        String studentNumber = getUserId().toUpperCase();

        if (ds.getStudent(studentNumber) == null) {
            setWarning(true);
            setWarningText("Unknown studentnumber.");
            cycle.activate(this);
            return;
        }

        Student s = ds.getStudent(studentNumber);
        if (!s.getEmail().equals(getEmail())) {
            setWarning(true);
            setWarningText("Studentnumber and email address don't match.");
            cycle.activate(this);
            return;
        }

        PasswordService passwordService = PasswordService.getInstance();
        String seed = passwordService.generatePassword(32);
        ds.removeRecoveryPageData(s.getStudentNumber());
        ds.setRecoveryPageData(passwordService.encrypt(seed), s.getStudentNumber());

        StringBuffer traklaMsg = new StringBuffer();
        traklaMsg.append("You have requested your log-in information for TRAKLA2.\n");
        /*
        traklaMsg.append("\n   Your studentnumber: ");
        traklaMsg.append(studentNumber); // Security threat to send both login and password in a unencrypted email
        traklaMsg.append("\n   Your password: ");
        traklaMsg.append(newPassword);
        traklaMsg.append("\n");
        */

        traklaMsg.append("\nFollow this link to generate a new password: https://" 
                + System.getProperty("https.server") + "/app?service=page&page=RecoverPassword&id=" 
                + seed + "\n");

        try {
            postMail(new String[] {s.getEmail()},
                "TRAKLA2 Lost password",
                traklaMsg.toString(), "trakla2-feedback@cs.hut.fi",
                System.getProperty("smtp.host"));
        } catch (MessagingException e) {
            e.printStackTrace();
            setWarning(true);
            setWarningText("Error sending the email: " + e.getMessage());
            cycle.activate(this);
            return;
        }

        // XXX Logging
        setWarning(true);
        setWarningText("Password sent.");
        cycle.activate(this);
    }

    public void detach() {
        super.detach();
    }

    private void postMail (String recipients[ ], String subject, String message, String from, String smtpHost)
        throws MessagingException {
        boolean debug = false;

        //Set the host smtp address
        Properties props = new Properties();
        props.put("mail.smtp.host", smtpHost);

        // create some properties and get the default Session
        Session session = Session.getDefaultInstance(props, null);
        session.setDebug(debug);

        // create a message
        Message msg = new MimeMessage(session);

        // set the from and to address
        InternetAddress addressFrom = new InternetAddress(from);
        msg.setFrom(addressFrom);

        InternetAddress[] addressTo = new InternetAddress[recipients.length];
        for (int i = 0; i < recipients.length; i++) {
            addressTo[i] = new InternetAddress(recipients[i]);
        }
        msg.setRecipients(Message.RecipientType.TO, addressTo);


        // Optional : You can also set your custom headers in the Email if you Want
        msg.addHeader("MyHeaderName", "myHeaderValue");

        // Setting the Subject and Content Type
        msg.setSubject(subject);
        msg.setContent(message, "text/plain");
        Transport.send(msg);
    }
}
