// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.util.Locale;

import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.IRequestCycle;
import cexer.*;

/** Provides dynamic behaviour for the Login page */
public abstract class MonitorLoginPage extends SecurePage {

    public abstract String getUserid();

    public abstract void setUserid(String value);

    public abstract String getPassword();

    public abstract void setPassword(String value);

    public abstract MonitorSession getMonitorSession();
    
    public abstract void setWarning(boolean state);
    
    
    public void formSubmit(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        if (getUserid() == null) { // added this check to assure compatibility
                                    // with tapestry 4
            setWarning(true);
            MonitorLoginPage next_page = (MonitorLoginPage)cycle.getPage("MonitorLogin");
            next_page.setPassword("");
            cycle.activate(next_page);
            return;
        }
        String userid = getUserid().toUpperCase();

        setWarning(false);
        DataStorage ds = DataStorage.getInstance();
        Student monitor = ds.getMonitor(userid);
        if (monitor == null || monitor.getPassword() == null
                || monitor.getAccessLevel() < Student.INSTRUCTOR_ACCESS
                || !monitor.matchPassword(getPassword())) {
            setWarning(true);
            MonitorLoginPage next_page = (MonitorLoginPage)cycle.getPage("MonitorLogin");
            next_page.setPassword("");
            cycle.activate(next_page);
            return;
        }
        session.setUserId(userid);
        session.setMonitor(monitor);
        session.setIsLoggedIn(true);
        cycle.getEngine().setLocale(new Locale("en", ""));
        // MonitorMainPage nextPage =
        // (MonitorMainPage)cycle.getPage("MonitorMain");
        //nextPage.updateRoundIndicator(visit.getCourse());
        cycle.activate("MonitorMain");
        return;
    }

    public void detach() {
        super.detach();
    }
}
