// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;


import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IAsset;

import cexer.*;
import cexer.util.Lock;
import cexer.util.LocaleSelectionModel;
import cexer.util.chart.ChartProvider;
import cexer.util.chart.ChartAsset;
import cexer.util.Logging;

import java.util.List;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Locale;
import java.awt.Paint;

import org.jCharts.Chart;
import org.jCharts.chartData.DataSeries;
import org.jCharts.chartData.AxisChartDataSet;
import org.jCharts.properties.AreaChartProperties;
import org.jCharts.properties.ChartProperties;
import org.jCharts.properties.AxisProperties;
import org.jCharts.properties.LegendProperties;
import org.jCharts.types.ChartType;
import org.jCharts.axisChart.AxisChart;
import org.jCharts.chartData.ChartDataException;
import org.jCharts.properties.StackedBarChartProperties;


public abstract class MonitorMainPage extends SecurePage implements ChartProvider {

    public abstract MonitorSession getMonitorSession();
    
    //private Course course; // persistent
    //private List studentResults; // persistent, depends on course
    private String student; // is required, used in finding student, not persistent
//      private boolean details; // persistent
//      private boolean grading; // persistent
//      private boolean activity; // persistent
//      private boolean gradeDistribution; // persistent
//      private boolean statistics; // persistent
//    private boolean warning;

    //private Statistics courseStatistics; // persistent, depends on course
    //private RoundStatistics roundStatistics; not required?
    //private ExerciseStatistics exerciseStatistics; not required?
    //private int[] gradeMinPoints; not required, minor adjustement to other methods
    //private int index, minPoints; // required
    //private String grade; not required
    //private boolean[] roundIndicator; // persistent, depends on course

//      private GregorianCalendar start; // persistent
//      private GregorianCalendar end; // persistent
//      private int activityViewMode; // persistent

    private IAsset pointDistributionImageAsset; // persistent? 
    private IAsset gradeDistributionImageAsset; // persistent?

    // charts
    public static String POINT_DISTRIBUTION = "points";
    public static String GRADE_DISTRIBUTION = "grades";
    public static String SERVER_ACTIVITY = "activity";

    // page state
    public static final int STATISTICS = 0;
    public static final int STUDENTS = 1;
    public static final int DETAILS = 2;
    public static final int ROUND_POINT_DISTRIBUTION = 3;
    
    public Course getCourse() {
  	MonitorSession session = getMonitorSession();
	Course c = session.getCourse();
  	if(c.getId() == DataStorage.DUMMY_COURSE) {
	    session.setCourse((Course)session.getStudent().getCourseList().get(0));
	    return session.getCourse();
	}
	return c;
//  	    if(course != null) {
//  		v.setCourse(course);
//  		return course;
//  	    } 
//  	}
//  	if(!c.equals(course)) {
//  	    setCourse(c);
//  	}
//  	return course;
    }

    public void setCourse(Course c) {
//  	course = c;
//  	Tapestry.fireObservedChange(this, "course", course);
//  	DataStorage ds = DataStorage.getInstance();
//  	Statistics s = ds.getStatisticsForCourse(getCourse().getId());
//  	setStatistics(s);
//  	List l = ds.getCourseResults(getCourse());
//  	setStudentResults(l);
//  	updateRoundIndicator(course);
    }


//      public void setDetails(boolean b) {
//  	details = b;
//  	Tapestry.fireObservedChange(this, "details", b);
//      }

//      public boolean isGrading() {
//    	return grading;
//      }

//      public void setGrading(boolean b) {
//  	grading = b;
//  	Tapestry.fireObservedChange(this, "details", b);
//      }

//      public boolean isGradeDistribution() {
//    	return gradeDistribution;
//      }

//      public void setGradeDistribution(boolean b) {
//  	gradeDistribution = b;
//  	Tapestry.fireObservedChange(this, "gradeDistribution", b);
//      }

//      public boolean isActivity() {
//    	return activity;
//      }

//      public void setActivity(boolean b) {
//  	activity = b;
//  	Tapestry.fireObservedChange(this, "activity", b);
//      }

    /**
     * @return true if the current view mode is course details
     */
    public boolean isShowDetails() {
	return getTabShowed() == DETAILS;
    }

    /**
     * @return true if the current view mode is round statistics 
     */
    public boolean isShowStatistics() {
	return getTabShowed() == STATISTICS;
    }
    
    /**
     * @return true if the current view mode is course students
     */
    public boolean isShowStudents() {
	return getTabShowed() == STUDENTS;
    }
    
    /**
     * @return true if the current view mode is round point distribution
     */
    public boolean isShowPointDistribution() {
        return getTabShowed() == ROUND_POINT_DISTRIBUTION;
    }

    /**
     * Gets the content of the student number textbox in student search
     * @return
     */
    public abstract String getStudentToFind();

    public abstract void setStudentToFind(String s);
    
    public abstract String getStudentNameToFind();
    
    public abstract void setStudentNameToFind(String name);
    
    public abstract String getStudentIdToFind();
    
    public abstract void setStudentIdToFind(String name);

    public abstract void setIndex(int i);
    
    public abstract int getIndex();
    
    public abstract int getEditingIndex();
    
    public abstract void setEditingIndex(int index);
    
    public abstract String getDescriptionLocale();
    
    /**
     * Get the content of the course description textarea
     * @return
     */
    public abstract String getChangedDescription();
    
    public abstract void setChangedDescription(String description);    

    public abstract String getEditingLocale();
    
    public abstract void setEditingLocale(String locale);  
    
    public abstract void setShowGradeDistribution(boolean state);
    
    public abstract boolean isShowGradeDistribution();

    public abstract void setActivity(boolean state);
    
    public abstract boolean isActivity();

    public abstract Locale getLocaleToAdd();

    public abstract MonitorSearchResultsPage getResultsPage();
    
    /**
     * Get the content of the course name textbox when adding a course
     * description in a new locale
     * @return
     */
    public abstract String getLocalizedCourseNameToAdd();
    
    abstract public int getTabShowed();
    
    abstract public void setTabShowed(int tabShowed);

    public abstract void setWarning(boolean state);
    
    public abstract void setSearchStringTooShort(boolean state);
    
    public abstract void setStudentNotFound(boolean state);

    public void detach() {
	super.detach();
	//course = null;
	//studentResults = null;
	student = null;
	//details = grading = activity = gradeDistribution = statistics = false;
	//index = minPoints = -1;
	//courseStatistics = null;
	//roundStatistics = null;
	//exerciseStatistics = null;
	//roundIndicator = null;
	//start = end = null;
	//activityViewMode = DAY;
	    setWarning(false);
    }
    
    /**
     * Clear the system cache. This removes cached courses, students and exercises.
     * @param cycle
     */
    public void clearCache(IRequestCycle cycle) {
	//Visit v = (Visit)getVisit();
	//System.out.println("Nrocourses: " + v.getStudent().getCourseList().size());
	DataStorage ds = DataStorage.getInstance();
	ds.emptyCache();
	//System.out.println("Nrocoursessss: " + v.getStudent().getCourseList().size());
	cycle.activate(this);
    }

    /**
     * Link to the all exercises page
     * @param cycle
     */
    public void allExercises(IRequestCycle cycle) {
	MonitorSession session = getMonitorSession();
	DataStorage ds = DataStorage.getInstance();
	session.setRound(ds.getAllExercises());
	//v.getRound().setAdditionalAppletParameters(new LinkedList());
	session.setCourse(ds.getCourse(DataStorage.DUMMY_COURSE));
	//v.getCourse().setAdditionalAppletParameters(new LinkedList());
	cycle.activate("MonitorAllExercises");
    }


    public void showCourseDetails(IRequestCycle cycle) {
	setTabShowed(DETAILS);
	cycle.activate(this);
    }

    public void showGradeDistribution(IRequestCycle cycle) {
        setShowGradeDistribution(!isShowGradeDistribution());
	cycle.activate(this);
    }

    public void showActivityLog(IRequestCycle cycle) {
        setActivity(!isActivity());
	cycle.activate(this);
    }

    public void showCourseStatistics(IRequestCycle cycle) {
	setTabShowed(STATISTICS);
	cycle.activate(this);
    }
 
    public void showCourseStudents(IRequestCycle cycle) {
	setTabShowed(STUDENTS);
	cycle.activate(this);
    }

    public void showPointDistribution(IRequestCycle cycle) {
        setTabShowed(ROUND_POINT_DISTRIBUTION);
        cycle.activate(this);
    }
    
    /**
     * Get a list of supported locales on the currently selected course
     * @return
     */
    public Iterator getLocales() {
        return getMonitorSession().getCourse().getSupportedLocaleIterator();
    }
    
    public boolean isEditingDescription() {
        if(getIndex() == getEditingIndex())
            return true;
        return false;    
    }
    
    /**
     * Activates the coursedescription editing mode
     * @param cycle
     */
    public void editDescription(IRequestCycle cycle) {
        Object[] params = cycle.getListenerParameters();
        DataStorage ds = DataStorage.getInstance();
        Lock l = ds.addLock(new Integer(getCourse().getId()), getMonitorSession());
        if(l == null) {
            cycle.activate(this);
            return;
        }
        setEditingIndex(((Integer)params[0]).intValue());
        setEditingLocale((String)params[1]);
        cycle.activate(this);
    }
    
    /**
     * Ends the coursedescription editing mode without saving changes
     * @param cycle
     */
    public void endEditDescription(IRequestCycle cycle) {
        releaseLock();       
        setEditingIndex(-1);
        setEditingLocale(null);
        cycle.activate(getPage());
    }
    
    /**
     * Gets the current course description for the editing textarea.
     * @return description
     */
    public String getDescription() {
        return getCourse().getLocalizedDescription(getDescriptionLocale());
    }
    
    /**
     * Saves the changed description to the session, for persistence over form rewind. The description is saved 
     * to the database by <b>modifyDescription</b>
     * @param description
     */
    public void setDescription(String description) {
        setChangedDescription(description);
    }
    
    /**
     * Saves a changed course description
     * @param cycle
     */
    public void modifyDescription(IRequestCycle cycle) {
        DataStorage ds = DataStorage.getInstance();
        if(getChangedDescription() != null && !getChangedDescription().equals("") && getEditingLocale() != null) {
            if(hasLock()) {
                /* System.out.println("MonitorMainPage: modifyDescription: getChangedDescription: '" 
                        + getChangedDescription() + "' getEditingLocale: '" + getEditingLocale()
                        + "' getCourse().getId: '" + getCourse().getId() + "' "); */
                boolean success = ds.updateCourseDescription(getChangedDescription(), getEditingLocale(), getCourse().getId());
                if(success) {
                    getCourse().setLocalizedDescription(getEditingLocale(), getChangedDescription());
                    //System.out.println("success");
                } else {
                    //setEditError(true);
                    //System.out.println("failure");
                    setEditingIndex(-1);
                    cycle.activate(this);
                    return;
                }
                releaseLock();
            }
        }
        setEditingIndex(-1);
        cycle.activate(this);        
    }
    
    /**
     * Releases a locked exercise description.
     * @return
     */
    private boolean releaseLock() {
        return checkLock(true);
        }

        private boolean checkLock(boolean b) {
        DataStorage ds = DataStorage.getInstance();
        MonitorSession session = getMonitorSession();
        Integer cid = new Integer(session.getCourse().getId());
        synchronized(this) {
            Lock l = ds.renewLock(cid, getMonitorSession());
            if(l == null) 
            return false;
            if(l.getId() == session) {
            if(b) {
                ds.removeLock(cid);
            }
            return true;
            }
        }
        return false;
        }

        private boolean hasLock() {
        return checkLock(false);
        }
    
    public Object[] createParameterArray(int parameter1, Object parameter2) {
        Object[] tempArray = new Object[2];
        tempArray[0] = new Integer(parameter1);
        tempArray[1] = parameter2;
        return tempArray;
    }


    public List getStudentResults() {
        return getMonitorSession().getStudentResults();

        //return studentResults;
    }

    /**
     * Searches for students in the database. The search is done by name or student
     * number, depending on where the search data is entered.
     * @param cycle
     */
    public void searchStudent(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        if((getStudentToFind() == null || getStudentToFind().length() < 2)
           && (getStudentNameToFind() == null || getStudentNameToFind().length() < 3)
           && (getStudentIdToFind() == null || getStudentIdToFind().length() < 2))
        {
            // Error situation
            setWarning(true);
            setSearchStringTooShort(true);
            return;
        }

        DataStorage ds = DataStorage.getInstance();
        List courses = session.getStudent().getAllCourses(); //XXX is the user really instructor on all these courses??
        List students = ds.searchStudents(getStudentNameToFind(), getStudentToFind(), getStudentIdToFind(), courses);

        if(students == null || students.isEmpty()) {
            setWarning(true);
            setStudentNotFound(true);
            return;
        }

        if(students.size() == 1) {
            // Only one student found
            Student stu = (Student)students.get(0);
            if(stu != null) {
                session.setMonitoredStudent(stu);
                List studentCourses = stu.getAllCourses();
                if(!studentCourses.contains(getMonitorSession().getCourse())) {
                    Course tempCourse = null;
                    ListIterator studentCourseIterator = studentCourses.listIterator();
                    while(studentCourseIterator.hasNext()) {
                        tempCourse = (Course)studentCourseIterator.next();
                        if(courses.contains(tempCourse)) {
                            session.setCourse(tempCourse);
                            break;
                        }
                    }
                }
                cycle.activate("MonitorStudent");
                return;
            }
        } else {
            // Many students found
            //session.setStudentsFound(students);
            //cycle.activate("MonitorSearchResults");

            MonitorSearchResultsPage resultsPage = getResultsPage();
            resultsPage.setResults(students);
            cycle.activate(resultsPage);

            return;
        }

        setWarning(true);
        setStudentNotFound(true);
        cycle.activate(this);
   }

    public void textResults(IRequestCycle cycle) {
        MonitorTextResultsPage next_page = (MonitorTextResultsPage) cycle.getPage("MonitorTextResults");
        next_page.setStudentResults(getStudentResults());

        cycle.activate(next_page);
    }

    public Statistics getStatistics() {
        return  getMonitorSession().getStatistics();
    }

    public List getRoundStatisticsList() {
        return getStatistics().getRounds();
    }

    public String getLocalizedCourseName() {
        return getCourse().getLocalizedName(getLocale().getLanguage());
    }

    public int[] getGradeMinPoints() {
        return getCourse().getGradeMinPoints();
    }

    public abstract int getMinPoints();

    public abstract void setMinPoints(int points);

    public String getPointsForGrade() {
	int[] gradeMinPoints = getCourse().getGradeMinPoints();
    int index = getIndex();
	if(index < gradeMinPoints.length - 1) {
	    return gradeMinPoints[index] + " - " + gradeMinPoints[index+1];
	}
	else { 
	    return gradeMinPoints[index] + " - " + getMonitorSession().getCourse().getMax();
	}
    }
    
    /**
     * @return true if the course has grading limits.
     */
    public boolean isGradeLimits() {
        if(getCourse().getGrades().length == 0 || getCourse().getGradeMinPoints().length == 0) 
            return false;
        return true;
    }
    

    public String getGrade() {
        return getCourse().getGrade(getMinPoints());
    }

    /**
     * @return Point distribution image asset
     */
    public IAsset getPointDistributionImage() {
	if(pointDistributionImageAsset == null) {
	    pointDistributionImageAsset = new ChartAsset(getRequestCycle(),this,POINT_DISTRIBUTION);
	}
	return pointDistributionImageAsset;
    }

    /**
     * @return Grade distribution image asset
     */
    public IAsset getGradeDistributionImage() {
	if(gradeDistributionImageAsset == null) {
	    gradeDistributionImageAsset = new ChartAsset(getRequestCycle(),this,GRADE_DISTRIBUTION);
	}
	return gradeDistributionImageAsset;
    }

    public Chart getChart(String chartName) throws ChartDataException {
	if (GRADE_DISTRIBUTION.equals(chartName)) {
	    return getGradeDistributionChart();
	} else { // the default functionality, includes GRADE_DISTRIBUTION
	    return getPointDistributionChart();
	}
    }

    private Chart getGradeDistributionChart() throws ChartDataException {
	// data for grade distribution.
	Course c = getCourse();
	int[] minPoints = c.getGradeMinPoints();
	// we add grade distribution only if the data is available
	if(minPoints != null && minPoints.length > 0) {
	// info for dataseries
	    String xtitle = "grades";
	    String ytitle = "Number of Students";
	    String title = "Distribution of grades";
	    String[] grades = c.getGrades();
	    String[] labels = new String[grades.length+1];
	    labels[0] = "0";
	    for(int j = 1; j < labels.length; j++) {
		labels[j] = grades[j-1];
	    }

	    DataSeries series = new DataSeries(labels,xtitle,ytitle,title);
	    List students = getStudentResults();
	    int[] gradeDistribution = new int[grades.length+1];
	    Iterator i = students.iterator();
	    // get the grades for all students on the course and 
	    // group the same grades together
	    while(i.hasNext()) {
		Student s = (Student)i.next();
		String grade = s.getGrade(c);
		if(grade.equals("0")) {
		    gradeDistribution[0]++;
		    continue;
		}
		for(int j = 0; j < grades.length; j++) {
		    if(grade.equals(grades[j])) {
			gradeDistribution[j+1]++;
			break;
		    }
		}
	    }

	    // create data for grade distribution and put the
	    // grade bars to the chart. Each grade bar is in the
	    // middle of the points that need to be gained for that
	    // grade.
	    double[][] data = new double[1][gradeDistribution.length];
	    for(int j = 0; j <= grades.length; j++) {
		data[0][j] = gradeDistribution[j];
	    }
	    // create properties, etc.
	    String[] labelTitles = new String[1];
	    labelTitles[0] = "Grade";
	    Paint[] paints = new Paint[] { java.awt.Color.red };
	    StackedBarChartProperties barChartProperties = new StackedBarChartProperties();
	    AxisChartDataSet axisData = new AxisChartDataSet(data, labelTitles,paints,ChartType.BAR,barChartProperties);
	    series.addIAxisPlotDataSet(axisData);
	    return createActualChart(series, 400, 300, false);
	}
	return null;
    }

    private Chart getPointDistributionChart() throws ChartDataException {
	Integer[] pointAmounts = getStatistics().getPointAmounts();

	// establish the length of the Xaxis.
	int length = pointAmounts.length;
	if(getCourse().getMax() > pointAmounts.length)
	    length = getCourse().getMax();
	String[] labels = new String[length];
	for(int i = 0; i < labels.length;i++) {
	    labels[i] = "" + i;
	}

	// info for dataseries
	String xtitle = "Points";
	String ytitle = "Number of Students";
	String title = "Distribution of Results";
	DataSeries series = new DataSeries(labels,xtitle,ytitle,title);

	// data for point distribution
	double[][] data = new double[1][length];
	for(int i = 0; i < pointAmounts.length;i++) {
	    data[0][i] = pointAmounts[i].doubleValue();
	}
	String [] labelTitles = { "Points" };
	Paint[] paints = { java.awt.Color.blue };
	AreaChartProperties aproperties = new AreaChartProperties();
	AxisChartDataSet axisData = new AxisChartDataSet(data,labelTitles,paints,ChartType.AREA,aproperties);
	series.addIAxisPlotDataSet(axisData);


	// chart creation itself
	return createActualChart(series);
    }


    private Chart createActualChart(DataSeries series) {
	return createActualChart(series, 800, 600, false);
    }

    private Chart createActualChart(DataSeries series, int width, int height, boolean horizontal) {
        ChartProperties chartProperties= new ChartProperties();
        chartProperties.setEdgePadding(20f);

        // FIXME: this does not compile -TA
        //AxisProperties axisProperties= new AxisProperties(horizontal);
        //axisProperties.getYAxisProperties().setShowGridLines(AxisTypeProperties.GRID_LINES_ALL);
        //axisProperties.getYAxisProperties().setShowTicks(AxisTypeProperties.TICKS_ALL);
        AxisProperties axisProperties = new AxisProperties();

        LegendProperties legendProperties= new LegendProperties();
        legendProperties.setNumColumns(2);

        Chart result = new AxisChart( series, chartProperties, axisProperties, legendProperties, width, height );
        return result;
    }

    public Chart getChart() throws ChartDataException {
	return getChart(null);
    }

    /**
     * @return true if the user has instructor rights on multiple courses.
     */
    public boolean isManyCourses() {
        MonitorSession session = getMonitorSession();
        List l = session.getStudent().getCourseList();
        return l.size() > 1;
    }
    
    public MonitorChartPopupPage chartPopup(IRequestCycle cycle, String chartName) {
        IAsset image = null;
        if(chartName.equals("grades"))
            image = getGradeDistributionImage();
        if(chartName.equals("points"))
            image = getPointDistributionImage();
        MonitorChartPopupPage nextPage = (MonitorChartPopupPage)cycle.getPage("MonitorChartPopup");
        nextPage.setImage(image);
        nextPage.setName(chartName);
        return nextPage;
    }
        
    /**
     * Tells if the course is missing a description in one (or more) of the locales specified in
     * DataStorage by getLocaleSelectionModel
     * @return true if course doesn't have descriptions in all available locales
     */
    public boolean isMissingLocales() {
        Iterator i = getLocales();
        DataStorage ds = DataStorage.getInstance();
        int size = 0;
        while(i.hasNext()) {
            size++;
            i.next();
        }
        if(size < ds.getLocaleSelectionModel().getOptionCount())
            return true;
        return false;
    }
    
    /**
     * Returns a LocaleSelectionModel containing the locales in which the course doesn't have a description.
     * @return LocaleSelectionModel with the missing locales
     */
    public LocaleSelectionModel getMissingLocaleModel() {
        Iterator i = getLocales();
        DataStorage ds = DataStorage.getInstance();
        List missingLocales = ds.getLocales();
        //System.out.println("ds.getLocales().size(): " + missingLocales.size());
        while(i.hasNext()) {
            missingLocales.remove(new Locale((String)i.next()));
        }
        return new LocaleSelectionModel(missingLocales);                                    
    }        
    
    /**
     * Adds a empty description in the selected locale
     * @param cycle
     */
    public void addLocale(IRequestCycle cycle) {
        if(getLocaleToAdd() == null || getLocalizedCourseNameToAdd() == null) {
            cycle.activate(this);
            return;
        }
        DataStorage ds = DataStorage.getInstance();
        boolean success = ds.addCourseData(getLocalizedCourseNameToAdd(), "", getLocaleToAdd().getLanguage(), getMonitorSession().getCourse().getId());
        if(success) {
            getMonitorSession().getCourse().addLocalizedData(getLocaleToAdd().getLanguage(), getLocalizedCourseNameToAdd(), "", "");            
        }
        cycle.activate(this);
    }
    
    
    /**
     * @return true if the course has a specified organizer
     */
    public boolean getHasOrganizer() {
        return getCourse().getOrganizer() != null;
    }

    /**
     * @return true if the course has a specified organization
     */
    public boolean getHasOrganization() {
        return getCourse().getOrganization() != null;
    }

    /**
     * @return string containing the localized organization name and acronym
     */
    public String getOrganizationName() {
        return getCourse().getOrganization().getLocalizedName(getLocale().getLanguage()) + " (" + getCourse().getOrganization().getLocalizedAcronym(getLocale().getLanguage()) + ")";
    }

    /**
     * Opens a popup with a student view of the current course 
     * @param cycle
     * @return Student interface main page object
     */
    public MainPage studentMainPagePopup(IRequestCycle cycle) {
        DataStorage ds = DataStorage.getInstance();
        Student tempStudent = ds.getStudent(getMonitorSession().getStudent().getId());

        if(tempStudent == null || !tempStudent.getCourseList().contains(getCourse())) {
            //Create dummy student for course
            tempStudent = new Student("MONITOR", "Monitor user");
            tempStudent.setCourse(getCourse());
        }
        //Login with tempStudent
        MainPage nextPage = (MainPage)cycle.getPage("Main");

        StudentSession studentSession = nextPage.getStudentSession();
        studentSession.setCourse(getCourse());
        //session.setWarning(false);
        studentSession.setIsLoggedIn(true);
        studentSession.setUserId(tempStudent.getId());
        studentSession.setName(tempStudent.getName());
        studentSession.setUsingMonitorUI(true);
        //cycle.getEngine().setLocale(new Locale(tempStudent.getLanguage()));
        cexer.util.Logging.info(this, "Monitor " + getMonitorSession().getUserId() + " logged in as a student from Monitor UI");

        return nextPage;
    }
}
