// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.IRequestCycle;
import cexer.*;
import cexer.util.Lock;

public abstract class MonitorModifyDescriptionPage extends SecurePage {

    String descr;

    public abstract String getTheLocale();
    public abstract void setTheLocale(String s);

    public abstract MonitorSession getMonitorSession();
    
    public String getDescription() {
        //System.out.println("getDescription(): locale: " + getTheLocale());
        return getMonitorSession().getExercise().getLocalizedDescription(getTheLocale());
    }

    public void setDescription(String s) {
	descr = s;
    }

    public void modify(IRequestCycle cycle) {
	setEditError(false);
	if(descr == null || descr == "") {
	    System.out.println("DID NOT MODIFY!");
	    cycle.activate("MonitorTestExercise");
	}
	DataStorage ds = DataStorage.getInstance();
        MonitorSession session = getMonitorSession();
	String locale = getTheLocale();
	    Exercise ex = session.getExercise();
	if(hasLock()) {
	    //System.out.println("MAKING MODIFICATION with locale: " + locale + " id " + session.getRound().getId() + " number " + ex.getNumber() + " template: " + ex.getExerciseTemplate().getId());
	    boolean success = ds.updateDescription(descr, locale,ex.getExerciseTemplate().getId());
	    if(success) {
		ExerciseTemplate et = ex.getExerciseTemplate();
		String name = et.getLocalizedName(getTheLocale());
		et.addLocalizedData(getTheLocale(),name,descr);
	    } else {
		setEditError(true);
		cycle.activate(this);
		return;
	    }
	    releaseLock();
	    cycle.activate("MonitorTestExercise");
	    return;
	} 
	setEditError(true);
	cycle.activate(this);
    }

    public void toMonitor(IRequestCycle cycle) {
	setEditError(false);
	releaseLock();
	cycle.activate("MonitorTestExercise");
    }

    public void toMain(IRequestCycle cycle) {
	setEditError(false);
	releaseLock();
	cycle.activate("MonitorMain");
    }

    public String getName() {
        String loc = getTheLocale();
        return getMonitorSession().getExercise().getLocalizedName(loc);
    }
    
    private boolean releaseLock() {
	return checkLock(true);
    }

    private boolean checkLock(boolean b) {
	DataStorage ds = DataStorage.getInstance();
    MonitorSession session = getMonitorSession();
	Integer eid = new Integer(session.getExercise().getExerciseTemplate().getId());
	synchronized(this) {
	    Lock l = ds.renewLock(eid, getMonitorSession());
	    if(l == null) 
		return false;
	    if(l.getId() == session) {
		if(b) {
		    ds.removeLock(eid);
		}
		return true;
	    }
	}
	return false;
    }

    private boolean hasLock() {
	return checkLock(false);
    }

    public void detach() {
	super.detach();
	descr = null;
    //releaseLock();
	//System.out.println("ModifyPage: detached");
    setEditError(false);
    }
    
    public boolean externalReleaseLock() {
        releaseLock();
        return true;
    }

    public abstract boolean isEditError();
   
    public abstract void setEditError(boolean b);

}
