// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import cexer.MonitorSession;
import java.util.List;
import java.util.ListIterator;
import java.util.LinkedList;
import org.apache.tapestry.annotations.Persist;

import org.apache.tapestry.IRequestCycle;

import cexer.Exercise;
import cexer.Student;
import cexer.Course;

public abstract class MonitorSearchResultsPage extends SecurePage {

    public abstract MonitorSession getMonitorSession();

    public abstract List getResults();
    public abstract void setResults(List results);

    public abstract String getRowStyle1();
    public abstract String getRowStyle2();
    public abstract int getIndex();
    public abstract int getSelectedIndex();
    public abstract void setSelectedIndex(int newSelectedIndex);
    public abstract Student getStudent();
    public abstract void setStudent(Student stu);

    public String getRowStyle() {
        if ((getIndex() % 2) == 1) {
            return getRowStyle2();
        } else {
            return getRowStyle1();
        }
    }

    public Object[] createParameterArray(int parameter1, int parameter2) {
        Object[] tempArray = new Object[2];
        tempArray[0] = new Integer(parameter1);
        tempArray[1] = new Integer(parameter2);
        return tempArray;
    }

    public List getAcceptedCourses() {
        List monitorCourses = getMonitorSession().getStudent().getAllCourses();
        List studentCourses = getStudent().getAllCourses();
        List acceptedCourses = new LinkedList();
        Course tempCourse = null;
        ListIterator studentCourseIterator = studentCourses.listIterator();
        while(studentCourseIterator.hasNext()) {
            tempCourse = (Course)studentCourseIterator.next();
            if(monitorCourses.contains(tempCourse)) {
                acceptedCourses.add(tempCourse);
            }
        }
        return acceptedCourses;
    }

    private List getAcceptedCourses(Student stu) {
        Student temp = getStudent();
        setStudent(stu);
        List courses = getAcceptedCourses();
        setStudent(temp);
        return courses;
    }

    public void setSelected(IRequestCycle cycle) {
        Object[] params = cycle.getListenerParameters();
        setSelectedIndex(((Integer)params[0]).intValue());
        cycle.activate(this);
    }

    public boolean isSelected() {
        return getIndex() == getSelectedIndex();
    }

    public void studentInfoAction(IRequestCycle cycle) {
        Object[] params = cycle.getListenerParameters();
        Student tempStudent = (Student)getResults().get(((Integer)params[0]).intValue());
        getMonitorSession().setMonitoredStudent(tempStudent);
        getMonitorSession().setCourse((Course)getAcceptedCourses(tempStudent).get(((Integer)params[1]).intValue()));

        cycle.activate("MonitorStudent");
    }
}
