// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.awt.Color;
import java.awt.Paint;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import org.apache.tapestry.IAsset;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.jCharts.Chart;
import org.jCharts.axisChart.AxisChart;
import org.jCharts.chartData.AxisChartDataSet;
import org.jCharts.chartData.ChartDataException;
import org.jCharts.chartData.DataSeries;
import org.jCharts.properties.AreaChartProperties;
import org.jCharts.properties.AxisProperties;
import org.jCharts.properties.AxisTypeProperties;
import org.jCharts.properties.BarChartProperties;
import org.jCharts.properties.ChartProperties;
import org.jCharts.properties.ClusteredBarChartProperties;
import org.jCharts.properties.LegendProperties;
import org.jCharts.properties.StackedBarChartProperties;
import org.jCharts.types.ChartType;

import cexer.Course;
import cexer.DataStorage;
import cexer.MonitorSession;
import cexer.Student;
import cexer.util.CourseSelectionModel;
import cexer.util.chart.*;

public abstract class MonitorServerActivityPage extends SecurePage implements
        ChartProvider {

    private IAsset serverActivityImageAsset;
    private IAsset dayAverageActivityImageAsset;

    public static String SERVER_ACTIVITY = "activity";
    
    public static Paint[] paints = new Paint[]{ Color.BLUE, Color.RED, Color.LIGHT_GRAY, Color.ORANGE, Color.CYAN, Color.GREEN, Color.BLACK, Color.DARK_GRAY, Color.YELLOW, Color.PINK, Color.MAGENTA };
    
    public abstract MonitorSession getMonitorSession();

    public abstract Course getCourse();

    public abstract boolean getShowResults();

    public abstract void setShowResults(boolean state);
    
    public abstract List getSelectedList();
    
    private Course getCurrentCourse() {
        return getMonitorSession().getCourse();
    }
    
    private Course getSelectedCourse() {
        return (Course) getSelectedList().get(0);
    }
    
    public IPropertySelectionModel getModel() {
        return new CourseSelectionModel(getMonitorSession().getStudent()
                .getCourseList());
    }

    public IAsset getServerActivityImage() {
        if (serverActivityImageAsset == null) {
            serverActivityImageAsset = new ChartAsset(getRequestCycle(), this,
                    SERVER_ACTIVITY);
        }
        return serverActivityImageAsset;
    }
    
    public IAsset getDayAverageActivityImage() {
        if (dayAverageActivityImageAsset == null) {
            dayAverageActivityImageAsset = new ChartAsset(getRequestCycle(), this,
                    new Integer(MonitorSession.DAY_AVERAGE).toString());
        }
        return dayAverageActivityImageAsset;
    }

    public Chart getChart(String chartName) throws ChartDataException {
        if (SERVER_ACTIVITY.equals(chartName)) {
            return getServerActivityChart();
        }
        if(Integer.parseInt(chartName) == MonitorSession.DAY_AVERAGE)
            return getServerActivityChart(MonitorSession.DAY_AVERAGE);        
        return null;
    }

    private Chart getServerActivityChart(int mode) throws ChartDataException {
        MonitorSession session = getMonitorSession();
        GregorianCalendar start = session.getStart();
        GregorianCalendar end = session.getEnd();
        DataStorage ds = DataStorage.getInstance();
        if (start == null || end == null) {
            start = new GregorianCalendar();
            end = new GregorianCalendar();
            start.setLenient(true);
            end.setLenient(true);
            session.setStart(start);
            session.setEnd(end);
            start.set(start.get(Calendar.YEAR), start.get(Calendar.MONTH),
                    start.get(Calendar.DAY_OF_MONTH), 0, 0, 0);
            end.set(end.get(Calendar.YEAR), end.get(Calendar.MONTH), end
                    .get(Calendar.DAY_OF_MONTH) + 1, 0, 0, 0);
        }
        String[] labels = null;
        GregorianCalendar counter = new GregorianCalendar();
        List courseList = getSelectedList();
        Iterator iter = courseList.iterator();
        int[][] activity = new int[courseList.size()][];
        for(int i = 0; iter.hasNext(); i++) 
            activity[i] = ds.getStudentActivityAverage((Course)iter.next());
        labels = new String[24];
        counter.set(Calendar.HOUR_OF_DAY, 0);
        counter.set(Calendar.MINUTE, 0);
        counter.set(Calendar.SECOND, 0);
        for (int i = 0; i < labels.length; i++) {
            labels[i] = "" + counter.get(Calendar.HOUR_OF_DAY) + ":00";
            counter.set(Calendar.HOUR_OF_DAY, counter
                    .get(Calendar.HOUR_OF_DAY) + 1);
        }
        String xtitle = "Time";
        String ytitle = "Number of Returned answers";
        String title = "Student activity distribution per hour";
        DataSeries series = new DataSeries(labels, xtitle, ytitle, title);
        
        double[][] data = new double[courseList.size()][24];
        String[] labelTitles = new String[courseList.size()];
        iter = courseList.iterator();
        for(int i = 0; iter.hasNext(); i++) {        
            for (int j = 0; j < activity[i].length; j++)
                data[i][j % 24] += (double) activity[i][j];
            labelTitles[i] = ((Course)iter.next()).getFullCourseName();    
        }
        Paint[] p = new Paint[courseList.size()];
        for(int i = 0; i < p.length; i++) {
            p[i] = paints[i%paints.length];
        }
        //BarChartProperties aproperties = new BarChartProperties();
        ClusteredBarChartProperties barChartProperties = new ClusteredBarChartProperties();
        AxisChartDataSet axisData = new AxisChartDataSet(data, labelTitles, p, ChartType.BAR_CLUSTERED, barChartProperties);
        series.addIAxisPlotDataSet(axisData);
        return createActualChart(series);            
    }
    
    /**
     * Gives the chart for the last 24 hours
     */
    private Chart getServerActivityChart() throws ChartDataException {
        MonitorSession session = getMonitorSession();
        GregorianCalendar start = session.getStart();
        GregorianCalendar end = session.getEnd();
        int mode = session.getActivityViewMode();
        List courseList = getSelectedList();
        if(courseList.size() == 0)
            throw new ChartDataException("No courses selected.");
        Iterator iter = courseList.iterator();
        // if defaults have not been set, set them
        if (mode == MonitorSession.NONE) {
            session.setActivityViewMode(MonitorSession.DAY);
            mode = MonitorSession.DAY;
        }
        DataStorage ds = DataStorage.getInstance();
        // if start and end times have not been set, set them
        // this should be done only when activity is viewed the first time.
        if (start == null || end == null) {
            start = new GregorianCalendar();
            end = new GregorianCalendar();
            start.setLenient(true);
            end.setLenient(true);
            session.setStart(start);
            session.setEnd(end);
            switch (mode) {
            case MonitorSession.WEEK:
                start.set(start.get(Calendar.YEAR), start.get(Calendar.MONTH),
                        start.get(Calendar.DAY_OF_MONTH) - 7, 0, 0, 0);
                end.set(end.get(Calendar.YEAR), end.get(Calendar.MONTH), end
                        .get(Calendar.DAY_OF_MONTH) + 1, 0, 0, 0);
                break;
            case MonitorSession.COURSE:
                Date d = null;
                Course tempCourse = null;
                for(int i = 0; iter.hasNext(); i++) {
                    tempCourse = (Course)iter.next();
                    if(d == null || tempCourse.getOpen().before(d))
                        d = tempCourse.getOpen();
                }
                if (d != null) {
                    start.setTime(d);
                } else {
                    iter = courseList.iterator();
                    for(int i = 0; iter.hasNext(); i++) {
                        tempCourse = (Course)iter.next();
                        if(d == null || ds.getDateOfFirstAnswer(tempCourse).before(d))
                            d = ds.getDateOfFirstAnswer(tempCourse);
                    }
                    if (d == null) {
                        // error, date of course start cannot be established
                        return null;
                    }
                    start.setTime(d);
                }
                iter = courseList.iterator();
                start.set(start.get(Calendar.YEAR), start.get(Calendar.MONTH),
                        start.get(Calendar.DAY_OF_MONTH), 0, 0, 0);
                end.set(end.get(Calendar.YEAR), end.get(Calendar.MONTH), end
                        .get(Calendar.DAY_OF_MONTH) + 1, end.get(Calendar.HOUR_OF_DAY), 0, 0);
                break;
            case MonitorSession.DAY_AVERAGE:
                start.set(start.get(Calendar.YEAR), start.get(Calendar.MONTH),
                        start.get(Calendar.DAY_OF_MONTH), 0, 0, 0);
                end.set(end.get(Calendar.YEAR), end.get(Calendar.MONTH), end
                        .get(Calendar.DAY_OF_MONTH) + 1, 0, 0, 0);
                break;
            case MonitorSession.DAY:
                start.set(start.get(Calendar.YEAR), start.get(Calendar.MONTH),
                        start.get(Calendar.DAY_OF_MONTH) - 1);
                break;
            default:
            }
        }
        SimpleDateFormat tf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        // System.out.println("fetching activity between " +
        // tf.format(start.getTime()) + " and " + tf.format(end.getTime()));
        int[][] activity = new int[courseList.size()][];
        String[] labels = null;
        GregorianCalendar counter = new GregorianCalendar();
        // fetch the raw data and create labeling for the x-axis
        switch (mode) {
        case MonitorSession.COURSE:
            end.set(end.get(Calendar.YEAR), end.get(Calendar.MONTH), end.get(Calendar.DAY_OF_MONTH) + 1, 0, 0, 0);
            for(int i = 0; iter.hasNext(); i++) {
                activity[i] = ds.getStudentActivityByDay((Course)iter.next(), start.getTime(), end.getTime());
                if(labels == null || labels.length < activity[i].length)
                    labels = new String[activity[i].length];
            }
            counter.setTime(start.getTime());
            for (int i = 0; i < labels.length; i++) {
                labels[i] = "" + counter.get(Calendar.DAY_OF_MONTH) + "."
                        + (counter.get(Calendar.MONTH) + 1);
                counter.set(Calendar.DAY_OF_YEAR, counter
                        .get(Calendar.DAY_OF_YEAR) + 1);
            }
            break;
        case MonitorSession.DAY_AVERAGE:
            for(int i = 0; iter.hasNext(); i++)
                activity[i] = ds.getStudentActivityAverage((Course)iter.next());
            labels = new String[24];
            counter.set(Calendar.HOUR_OF_DAY, 0);
            counter.set(Calendar.MINUTE, 0);
            counter.set(Calendar.SECOND, 0);
            for (int i = 0; i < labels.length; i++) {
                labels[i] = "" + counter.get(Calendar.HOUR_OF_DAY) + ":00";
                counter.set(Calendar.HOUR_OF_DAY, counter
                        .get(Calendar.HOUR_OF_DAY) + 1);
            }
            break;
        case MonitorSession.WEEK:
            for(int i = 0; iter.hasNext(); i++) {
                activity[i] = ds.getStudentActivity((Course)iter.next(), start.getTime(), end
                        .getTime(), DataStorage.MILLIS_IN_HOUR * 4);
                if(labels == null || labels.length < activity[i].length)
                    labels = new String[activity[i].length];
            }
            counter.setTime(start.getTime());
            for (int i = 0; i < labels.length; i++) {
                if (counter.get(Calendar.HOUR_OF_DAY) == 0) {
                    labels[i] = "" + counter.get(Calendar.DAY_OF_MONTH) + "."
                            + (counter.get(Calendar.MONTH) + 1);
                    // " " + counter.get(Calendar.HOUR_OF_DAY) + ":00";
                } else {
                    labels[i] = " "; // "" +
                    // counter.get(Calendar.HOUR_OF_DAY) +
                    // ":00";
                }
                counter.set(Calendar.HOUR_OF_DAY, counter
                        .get(Calendar.HOUR_OF_DAY) + 4);
            }
            break;
        case MonitorSession.DAY:
        default:
            for(int i = 0; iter.hasNext(); i++)
                activity[i] = ds.getStudentActivityByHour((Course)iter.next(),start.getTime(), end.getTime());
            labels = new String[activity[0].length];
            counter.setTime(start.getTime());
            for (int i = 0; i < labels.length; i++) {
                if (counter.get(Calendar.HOUR_OF_DAY) == 0) {
                    labels[i] = "" + counter.get(Calendar.DAY_OF_MONTH) + "."
                            + (counter.get(Calendar.MONTH) + 1) + " "
                            + counter.get(Calendar.HOUR_OF_DAY) + ":00";
                } else {
                    labels[i] = "" + counter.get(Calendar.HOUR_OF_DAY) + ":00";
                }
                counter.set(Calendar.HOUR_OF_DAY, counter
                        .get(Calendar.HOUR_OF_DAY) + 1);
            }
        }
        // info for dataseries
        String xtitle = "Time";
        String ytitle = "Number of Returned answers";
        String title = "";
        if (mode == MonitorSession.DAY_AVERAGE) {
            title = "Student activity distribution per hour";
        } else {
            title = "Student activity between " + tf.format(start.getTime())
                    + " and " + tf.format(end.getTime());
        }
        DataSeries series = new DataSeries(labels, xtitle, ytitle, title);

        // process the data to required form. If we are looking at the data
        // distribution between hours of a day we need different methodology
        double[][] data = null;
        String[] labelTitles = new String[courseList.size()];
        Course tempCourse = null;
        if (mode == MonitorSession.DAY_AVERAGE) {
            data = new double[courseList.size()][24];
            iter = courseList.iterator();
            for(int i = 0; iter.hasNext(); i++) {
                tempCourse = (Course)iter.next();
                for (int j = 0; j < activity[i].length; j++) 
                    data[i][j % 24] += (double) activity[i][j];
                labelTitles[i] = tempCourse.getCourseCode() + " " + tempCourse.getLocalizedName(getLocale().getLanguage());    
                }
        } else {
            //data = new double[courseList.size()][activity.length];
            data = new double[courseList.size()][labels.length]; //labels.length = length of the longest activity-dataset
            iter = courseList.iterator();
            for(int i = 0; iter.hasNext(); i++) { 
                tempCourse = (Course)iter.next();
                for (int j = 0; j < activity[i].length; j++)
                    data[i][j] = (double) activity[i][j];
                labelTitles[i] = tempCourse.getCourseCode() + " " + tempCourse.getLocalizedName(getLocale().getLanguage());               
            }
        }
        Paint[] p = new Paint[courseList.size()];
        for(int i = 0; i < p.length; i++) {
            p[i] = paints[i%paints.length];
        }
        ClusteredBarChartProperties aproperties = new ClusteredBarChartProperties();
        if(mode == MonitorSession.COURSE)
            aproperties.setShowOutlinesFlag(false);
        AxisChartDataSet axisData = new AxisChartDataSet(data, labelTitles, p, ChartType.BAR_CLUSTERED, aproperties);
        series.addIAxisPlotDataSet(axisData);
        return createActualChart(series);
    }

    private Chart createActualChart(DataSeries series) {
        return createActualChart(series, 800, 600, false);
    }

    private Chart createActualChart(DataSeries series, int width, int height, boolean horizontal) {
        ChartProperties chartProperties = new ChartProperties();
        chartProperties.setEdgePadding(20f);

        // FIXME: this does not compile -TA
        //AxisProperties axisProperties = new AxisProperties(horizontal);
        //axisProperties.getYAxisProperties().setShowGridLines(AxisTypeProperties.GRID_LINES_ALL);
        //axisProperties.getYAxisProperties().setShowTicks(AxisTypeProperties.TICKS_ALL);
        AxisProperties axisProperties = new AxisProperties();

        LegendProperties legendProperties = new LegendProperties();
        legendProperties.setNumColumns(2);

        Chart result = new AxisChart(series, chartProperties, axisProperties,
                legendProperties, width, height);
        return result;
    }

    public Chart getChart() throws ChartDataException {
        return getChart(null);
    }

    public void previousActivity(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        GregorianCalendar start = session.getStart();
        GregorianCalendar end = session.getEnd();
        switch (session.getActivityViewMode()) {
        case MonitorSession.NONE:
            break;
        case MonitorSession.DAY:
            start
                    .set(Calendar.DAY_OF_YEAR,
                            start.get(Calendar.DAY_OF_YEAR) - 1);
            end.set(Calendar.DAY_OF_YEAR, end.get(Calendar.DAY_OF_YEAR) - 1);
            break;
        case MonitorSession.WEEK:
            start
                    .set(Calendar.DAY_OF_YEAR,
                            start.get(Calendar.DAY_OF_YEAR) - 7);
            end.set(Calendar.DAY_OF_YEAR, end.get(Calendar.DAY_OF_YEAR) - 7);
            break;
        case MonitorSession.COURSE:
        case MonitorSession.DAY_AVERAGE:
        default:
            // there's nothing to be done
            return;
        }
        cycle.activate(this);
        return;
    }

    public void nextActivity(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        GregorianCalendar start = session.getStart();
        GregorianCalendar end = session.getEnd();
        switch (session.getActivityViewMode()) {
        case MonitorSession.NONE:
            break;
        case MonitorSession.DAY:
            start
                    .set(Calendar.DAY_OF_YEAR,
                            start.get(Calendar.DAY_OF_YEAR) + 1);
            end.set(Calendar.DAY_OF_YEAR, end.get(Calendar.DAY_OF_YEAR) + 1);
            break;
        case MonitorSession.WEEK:
            start
                    .set(Calendar.DAY_OF_YEAR,
                            start.get(Calendar.DAY_OF_YEAR) + 7);
            end.set(Calendar.DAY_OF_YEAR, end.get(Calendar.DAY_OF_YEAR) + 7);
            break;
        case MonitorSession.COURSE:
        case MonitorSession.DAY_AVERAGE:
        default:
            // there's nothing to be done
            return;
        }
        cycle.activate(this);
        return;

    }

    public void dayMode(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        GregorianCalendar start = session.getStart();
        GregorianCalendar end = session.getEnd();
        end.setTime(start.getTime());
        end.set(Calendar.DAY_OF_YEAR, end.get(Calendar.DAY_OF_YEAR) + 1);
        session.setActivityViewMode(MonitorSession.DAY);
        cycle.activate(this);
    }

    public void weekMode(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        GregorianCalendar start = session.getStart();
        GregorianCalendar end = session.getEnd();
        start.set(start.get(Calendar.YEAR), start.get(Calendar.MONTH), start
                .get(Calendar.DAY_OF_MONTH), 0, 0, 0);
        end.set(start.get(Calendar.YEAR), start.get(Calendar.MONTH), start
                .get(Calendar.DAY_OF_MONTH) + 7, 0, 0, 0);
        session.setActivityViewMode(MonitorSession.WEEK);
        cycle.activate(this);
    }

    public void courseMode(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        GregorianCalendar start = session.getStart();
        GregorianCalendar end = session.getEnd();
        Date d = null;
        Course tempCourse = null;
        Iterator iter = getSelectedList().iterator();
        for(int i = 0; iter.hasNext(); i++) {
            tempCourse = (Course)iter.next();
            if(d == null || tempCourse.getOpen().before(d))
                d = tempCourse.getOpen();
        }
        if (d != null) {
            start.setTime(d);
        } else {
            iter = getSelectedList().iterator();
            DataStorage ds = DataStorage.getInstance();
            for(int i = 0; iter.hasNext(); i++) {
                tempCourse = (Course)iter.next();
                if(d == null || ds.getDateOfFirstAnswer(tempCourse).before(d))
                    d = ds.getDateOfFirstAnswer(tempCourse);
            }
            if (d == null) {
                // error, date of course start cannot be established
                //return null;
            }
            start.setTime(d);
        }
        end.setTime(new Date());
        session.setActivityViewMode(MonitorSession.COURSE);
        cycle.activate(this);
    }

    public void dayAverageMode(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        session.setActivityViewMode(MonitorSession.DAY_AVERAGE);
        cycle.activate(this);

    }

    public String getLocalizedName() {
        return getCourse().getCourseCode() + " "
                + getCourse().getLocalizedName(getLocale().getLanguage());
    }

    public void compareCourses(IRequestCycle cycle) {
        setShowResults(true);
        cycle.activate(this);
    }

    public void selectCourses(IRequestCycle cycle) {
        setShowResults(false);
        cycle.activate(this);
    }

    public void detach() {
        super.detach();
        setShowResults(false);
    }

}
