// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.util.List;
import java.util.LinkedList;
import java.util.ListIterator;

import java.util.Locale;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.valid.ValidationDelegate;
import cexer.*;
import cexer.util.Checker;
import cexer.util.CourseSelectionModel;
import cexer.util.Logging;
import cexer.util.LocaleSelectionModel;

public abstract class TestRegisterPage extends SecurePage {

    /*
    private String pwd;

    private String pwd2;

    private String email;

    private Locale lang = new Locale(Student.DEFAULT_LANGUAGE);
    */

    /*
     * Tapesty 4 application state object. Fetches the injected student-session
     * object
     */

    public abstract StudentSession getStudentSession();

    public abstract void setWarning(boolean state);

    public abstract void setWarningText(String warningText);

    public abstract String getUserId();

    public abstract String getName();

    public abstract void setName(String name);

    public abstract void setEmail(String email);

    public Course getCourse() {
        return DataStorage
                .getInstance()
                .getCourse(Integer.parseInt(System.getProperty("testcourseid")));
    }

    public void formSubmit(IRequestCycle cycle) {
        setWarning(false);
        ValidationDelegate delegate = (ValidationDelegate) getBeans().getBean("delegate");
        if (delegate.getHasErrors()) {
            setPassword("");
            setPassword2("");
            //visit.setUsername(null);
            cycle.activate(this);
            return;
        }

        DataStorage ds = DataStorage.getInstance();
        String studentNumber = getUserId().toUpperCase();
        if (ds.getStudent(studentNumber) != null) {
            String msg = "User " + getUserId() + " already registered";
            clearFields();
            setWarning(true);
            setWarningText(msg);
            cycle.activate(this);
            return;
        }

        String checker = getCourse().getCheckerClass();
        if (Checker.check(studentNumber, checker)) {
            if (!getPassword().equals(getPassword2())) {
                setPassword("");
                setPassword2("");
                setWarning(true);
                setWarningText("Passwords do not match");
                cycle.activate(this);
                return;
            }

            if (getPassword().length() > 12) {
                setPassword("");
                setPassword2("");
                setWarning(true);
                setWarningText("Password too long (max 12 characters).");
                cycle.activate(this);
                return;
            }

            Student student = new Student(studentNumber, getName());
            student.setName(getName());
            student.setPassword(getPassword());
            student.setEmail(getEmail());
            student.setStudentNumber(studentNumber);
            String lang = getLanguage().getLanguage();
            student.setLanguage(lang);
            boolean ok = ds.addStudent(student);

            if (!ok) {
                clearFields();
                setWarning(true);
                setWarningText("Error when adding student!");
                cycle.activate(this);
                return;
            }

            student.addCourse(getCourse());
            ds.registerStudentToCourse(studentNumber, getCourse().getId());
            getEngine().setLocale(getLanguage());
            StudentSession session = getStudentSession();
            session.clearFields();
            //session.setWarning(false);
            session.setIsLoggedIn(true);
            session.setUserId(studentNumber);
            //visit.setUsername(studentNumber); // use the uppercase name
            cexer.util.Logging.info("Student " + studentNumber + " registered to the system.");
            cycle.activate("Main");
        } else {
            clearFields();
            setWarning(true);
            setWarningText("Invalid username");
            cycle.activate(this);
        }
    }

    private void clearFields() {
        StudentSession session = getStudentSession();
        session.clearFields();
        setPassword("");
        setPassword2("");
    }

    public String getNameText() {
        return getMessage("name");
    }

    public String getNumberText() {
        return getMessage("number");
    }

    public void detach() {
        super.detach();
        /*
          
         pwd = null; 
        pwd2 = null;
        email = null;
        lang = new Locale(Student.DEFAULT_LANGUAGE);
        */
    }


    public abstract void setPassword(String value);

    public abstract String getPassword();

    public abstract void setPassword2(String value);

    public abstract String getPassword2();

    public abstract String getEmail();

    public abstract Locale getLanguage();

    
    public LocaleSelectionModel getLocaleModel() {
        LocaleSelectionModel lsm = DataStorage.getInstance()
                .getLocaleSelectionModel();
        return lsm;
    }
}

