// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util;

/**
 * Abstract superclass for all studentnumber checkers. This class can
 * also be used to instantiate and check the studentnumber using some
 * other class by using the method <code>check</code>.
 */
public abstract class Checker {

    /** Returns true, if the given studentnumber is valid and false if not */
    public abstract boolean isCorrectStudentNumber(String studentNumber);

    /** Checks the given studentnumber using the checker given as parameter.
     * The parameter checker should be the full classname of the Checker. */
    public static boolean check(String studentNumber, String checker) {
	if (checker.equalsIgnoreCase("none"))
	    return true;
	try {
	    Class c = Class.forName(checker);
	    Checker check = (Checker)c.newInstance();
	    return check.isCorrectStudentNumber(studentNumber);
	} catch (ClassNotFoundException e) {
	    e.printStackTrace();
	    Logging.error("Checker " + checker + " not found!");
	    return false;
	} catch (ClassCastException e) {
	    e.printStackTrace();
	    Logging.error("Invalid Checker " + checker + "!");
	    return false;
	} catch (Exception e) {
	    e.printStackTrace();
	    Logging.error("Checker " + checker + " could not be instantiated!");
	    return false;
	} 
    }
}
