// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util;

/**
 * Class that can be used to check if a studentnumber is valid 
 * Helsinki University of Technology styled number. This is a 
 * string of 5 digits plus one character.
 */
public class HUTChecker extends Checker {
    private char[] letters = new char[] {'A','B','C','D','E','F','H','J','K','L','M','N','P','R','S','T','U','V','W'}; 

    public HUTChecker() {
	//System.out.println("Created a HUTChecker");
    }

    public boolean isCorrectStudentNumber(String studentNumber) {
	//System.out.println(studentNumber.substring(0, studentNumber.length() - 1) + "-"+studentNumber.charAt(studentNumber.length()-1));
	// maximum student number size (at the moment) is 5 digits + 1 letter
	if (studentNumber.length() > 6)
	    return false;
	int nro = -1;	
	int i = 0;
	// for-loop stores in i the index of the first non-zero number in the string
	for(;studentNumber.charAt(i) == '0' && i < studentNumber.length();i++);
	try {
	    nro = Integer.decode(studentNumber.substring(i, studentNumber.length() - 1)).intValue();
	} catch(NumberFormatException e) {
	    Logging.info(this, "Error checking studentnumber "+studentNumber, e);
	    return false;
	}
	// if default has not been changed, it cannot be a valid student number
	if(nro < 0) {
	    return false;
	}
	nro = nro % 19;
	if(letters[nro] == studentNumber.charAt(studentNumber.length()-1)) {
	    return true;
	}
	return false;
    }
}
