// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * 
 */
package cexer.util;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Random;
import sun.misc.BASE64Encoder;

/**
 * Class for creating and encrypting passwords.
 * 
 * @author erik
 *
 */
public final class PasswordService {

    /**
     * Singleton instance of the password service.
     */
    private static PasswordService serviceInstance;
    
    /**
     * List of characters. Used for password generation.
     */
    private static char[] chars = { 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h',
        'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
        'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 
        'K', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 
        'Z', '1', '2', '3', '4', '5', '6', '7', '8', '9' };


    /**
     * Private constructor.
     */
    private PasswordService() { } 
    
    /**
     * Gets the <code>PasswordService</code> instance.
     * 
     * @return PasswordService instance
     */
    public static synchronized PasswordService getInstance() {
        if(serviceInstance == null) 
            serviceInstance = new PasswordService();
        return serviceInstance;
    }

    /**
     * Encrypt a string using the SHA algorithm.
     * 
     * @param text text to encrypt
     * @return encrypted text
     */
    public synchronized String encrypt(String text) {

        MessageDigest md = null;

        try {
            md = MessageDigest.getInstance("SHA");
        } catch(NoSuchAlgorithmException e) {
            e.printStackTrace();
        }

        try {
            md.update(text.getBytes("UTF-8"));
        } catch(UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        byte raw[] = md.digest();
        String output = (new BASE64Encoder()).encode(raw);
        return output;
    }

    /**
     * Generate a password of random uppercase and lowercase
     * charachters and numbers.
     * 
     * @param passwordLength length of the password to generate
     * @return generated password
     */
    public synchronized String generatePassword(int passwordLength) {
        Random rand = new Random();
        String password = "";
        for (int i = 0; i < passwordLength; i++) {
          password += chars[rand.nextInt(chars.length)];
        }
        return password;
    }    
}
