// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.tapestry.INamespace;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.PageRedirectException;
import org.apache.tapestry.resolver.ISpecificationResolverDelegate;
import org.apache.tapestry.spec.IComponentSpecification;


public class SpecificationResolverErrorDelegate implements ISpecificationResolverDelegate {

    /*
     * Throws a Page Redirect exception to report that a requested page
     * specification could not be found. The target error page name is
     * the value configured in the application specification file or the
     * default given by DEFAULT_SPECIFICATION_ERROR_PAGE.
     */
    public IComponentSpecification findPageSpecification(IRequestCycle cycle, INamespace namespace, String simplePageName) {

        cexer.util.Logging.info(this, "findPageSpecification()");
        
        String errorPageName = cycle.getEngine().getSpecification().getProperty("org.apache.tapestry.home-page");

        // Handle special case where the error page itself cannot be found
        if (simplePageName.equals(errorPageName)) {
            return null;
        }

        // Store information about the error in the request similar to Tomcat
        HttpServletRequest request = cycle.getRequestContext().getRequest();
        HttpServletResponse response =  cycle.getRequestContext().getResponse();

        request.setAttribute("javax.servlet.error.status_code", new Integer(HttpServletResponse.SC_NOT_FOUND));
        request.setAttribute("javax.servlet.error.message", "Unable to find the requested page");
        request.setAttribute("javax.servlet.error.request_uri", request.getRequestURI());
        request.setAttribute("javax.servlet.error.servlet_name", request.getServerName());
        request.removeAttribute("javax.servlet.error.exception_type");

        // Set the http response code to 404
        response.setStatus(HttpServletResponse.SC_NOT_FOUND);

        throw new PageRedirectException(simplePageName, null, null, "Home");
    }

    public IComponentSpecification findComponentSpecification(IRequestCycle cycle, INamespace namespace, String type) {
        return null;
    }
}
