// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util;

import java.sql.*;
import java.util.*;
import java.io.*;


public class Trakla2SupportGenerator {

    /** 
     * Opens a database connection. The connection must be closed
     * using closeDatabaseConnection. 
     */
    private static Connection getDatabaseConnection() {
	try { 
	    String driver = System.getProperty("jdbc.driver", "org.postgresql.Driver");
	    // Needs to be done on the first time only??
	    Class.forName(driver);


	    String dbUrl = System.getProperty("db.url", "jdbc:postgresql:trakla2");
	    String username = System.getProperty("db.username", "trakla2");
	    String passwd = System.getProperty("db.password", "10945086");
	    return DriverManager.getConnection(dbUrl, username, passwd); 
	} catch (ClassNotFoundException e) {
	    System.out.println("Database driver not found");
	    e.printStackTrace();
	    return null;
	} catch (SQLException e) {
	    System.out.println("Database connection couldn't be opened");
	    e.printStackTrace();
	    return null;
	}
    }

    /** 
     * Returns the ResultSet of the given query or null if an 
     * exception occurs.
     */
    private static ResultSet executeQuery(String query) throws SQLException {
	Connection con = getDatabaseConnection();
	Statement stmt = con.createStatement();
	ResultSet result = stmt.executeQuery(query);
	con.close();
	return result;
    }

    /**
     * This is probably the most ugly method I've ever written. (Which says a lot). -Erik
     * 
     * The selected page locale can be provided as an argument to the main function.
     * 
     * @param args locale
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
	String lang = "en";
	if (args.length > 0)
	    lang = args[0];
	String query = "SELECT name, description, eid FROM exercisedescription WHERE lang='" + lang + "' ORDER BY eid;";
	ResultSet rs = executeQuery(query);
	Hashtable names = new Hashtable();
	Hashtable descr = new Hashtable();
	while (rs.next()) {
	    int eid = rs.getInt("eid");
	    names.put(new Integer(eid), rs.getString("name"));
	    descr.put(new Integer(eid), rs.getString("description"));
	    //System.out.println(eid + " " + rs.getString("name"));
	}
	Enumeration eids = names.keys();
    ExerciseDescriptionParser parser = null;
    HashMap codeBlocks = null;
    HashMap exerciseFullNames = new HashMap();
    HashMap eidToExerciseClass = new HashMap();
    LinkedList exerciseNames = new LinkedList();
    LinkedList codeBlockNames = null;
    
	while (eids.hasMoreElements()) {
	    int eid = ((Integer)eids.nextElement()).intValue();
	    query = "SELECT name, value FROM appletparameter WHERE type='e' AND id=" + eid + ";";
	    rs = executeQuery(query);
	    Hashtable params = new Hashtable();
	    String eClass = null;
	    while (rs.next()) {
		String name = rs.getString("name");
		String value = rs.getString("value");
		if (name.equals("ex"))
		    eClass = value;
		params.put(name, value);
	    }
	    if(eClass != null) {
        String seClass = eClass.substring(eClass.lastIndexOf(".") + 1);
	    System.out.println(seClass);
	    query = "SELECT code, height, width, codebase, archive FROM exercise WHERE exercise.id=" + eid +";";
	    rs = executeQuery(query);
	    exerciseNames.add(seClass);
        eidToExerciseClass.put(new Integer(eid).toString(), seClass);
        exerciseFullNames.put(seClass, names.get(new Integer(eid)));
	    FileWriter fw = new FileWriter(new File("support/" + seClass + ".html"));
	    while (rs.next()) {
		fw.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n" +
                "<html>\n<head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n" +
                "<title>TRAKLA2 - " + names.get(new Integer(eid)) + "</title>" +
                        "<link rev=\"made\" href=\"mailto:matrix@cs.hut.fi\">" +
                        "<link rel=\"stylesheet\" href=\"trakla2.css\" type=\"text/css\">" +
                        "</head>\n<body>");
        fw.write("<table width=\"100%\" class=\"borderTable\">" +
                "<tr><td class=\"borderlink\"><h1 class=\"applicationName\">TRAKLA2</h1></td>" +
                "<td class=\"borderlink\"><a href=\"index.html\">Index</a> &nbsp;</td>" +
                "<td class=\"borderlink\"><a href=\"http://www.cs.hut.fi/Research/TRAKLA2/\">Research</a> &nbsp;</td>" +
                "<td class=\"borderlink\"><a href=\"http://www.cs.hut.fi/Research/TRAKLA2/support.shtml\">Support</a></td>" +
                "</tr></table>\n");
		fw.write("<div class=\"cexerpage\">" +
                "<h2>" + names.get(new Integer(eid)) + "</h2><div class=\"mainTable\">\n" +
                        "<script type=\"text/javascript\" src=\"ExercisePageTabs.script\"></script>\n");
        
        parser = new ExerciseDescriptionParser(new BufferedReader(new StringReader((String)descr.get(new Integer(eid)))));
        codeBlocks = parser.getCodeBlocks();
        codeBlockNames = parser.getCodeBlockNames();
                
        if(parser.getTask().equals(""))
            fw.write("<div class=\"descriptionBox\">" + (String)descr.get(new Integer(eid)) + "</div>\n");            
        else {                    
            fw.write("<table border=\"0\" width=\"100%\" class=\"exerciseTable\" cellspacing=\"0\" cellpadding=\"0\">" +
                    "<tr><td valign=\"top\" class=\"tabrow\" colspan=\"2\" width=\"*\">\n" +
                    "<a href=\"#\" id=\"hideLink\" onclick=\"hideExerciseTabs();return false;\">Hide text</a>" +
                    "<a href=\"#\" id=\"showLink\" onclick=\"showExerciseTabs();return false;\" style=\"display: none\">Show text</a>&nbsp;" +                
                    "<a href=\"#\" id=\"hideCodeLink\" onclick=\"hideCodeTabs();return false;\">Hide pseudo-code</a>" +
                    "<a href=\"#\" id=\"showCodeLink\" onclick=\"showCodeTabs();return false;\" style=\"display: none\">Show pseudo-code</a>" +
                    "</td></tr>\n" +
                    "<tr><td colspan=\"2\">\n" +
                    "<div id=\"instructions\"> <div class=\"header\"><ul>\n" +
                    "<li id=\"task-tab\" class=\"current\">" +
                    "<a href=\"#\" onclick=\"showTab(this, 'task'); return false;\"><span class=\"tab-link\">Task</span></a></li>\n");
            if(!parser.getInstructions().equals(""))
                fw.write("<li id=\"instructions-tab\"><a href=\"#\" onclick=\"showTab(this, 'instructions'); return false;\"><span class=\"tab-link\">Instructions</span></a></li>\n");
            fw.write("</ul></div>");
            fw.write("<div class=\"exerciseTabContent\"><div id=\"task-text\">"
                    + parser.getTask() + "</div>\n");
            if(!parser.getInstructions().equals(""))
                fw.write("<div id=\"instructions-text\" style=\"display:none\">"
                        + parser.getInstructions() + "</div>\n");
            fw.write("</div></div></td></tr>\n");
            
            if(codeBlockNames.size() > 0) {
                fw.write("<td width=\"*\" valign=\"top\" id=\"code-td\" class=\"exerciseTab\"><div id=\"code\">"
                        + "<div class=\"header\"><ul>\n");
                for(int i = 0; i < codeBlockNames.size();i++) {
                    fw.write("<li id=\""+ codeBlockNames.get(i) + "\"");
                    if(i == 0)
                        fw.write(" class=\"current\"");
                    fw.write(">" +
                            "<a onclick=\"showTab(this, '" + codeBlockNames.get(i) + "'); return false;\" href=\"#\">" +
                            "<span class=\"tab-link\">"+ codeBlockNames.get(i) +"</span>" +
                            "</a></li>\n");
                }
                fw.write("</ul></div>" +
                        "<div class=\"exerciseTabContent\">\n");
                for(int i = 0; i < codeBlockNames.size();i++) {
                    fw.write("<div style=\"");
                    if(i != 0)
                        fw.write("display: none");
                    fw.write("\" id=\"" + ((String)codeBlockNames.get(i)).toLowerCase() + "-text\">"
                            + codeBlocks.get(codeBlockNames.get(i))
                            + "</div>");
                }
                fw.write("</div></div></td>\n\n");
            }
        
            fw.write("<td width=\"551px\" valign=\"top\" align=\"right\" class=\"exerciseTab\">");
        }
        
		fw.write("<div class=\"exerciseAppletBorder\"><applet archive=\"" + rs.getString("archive") +
			 "\" code=\"applications.exerciseapplet.ExerciseApplet.class" +
			 "\" width=\"" + rs.getInt("width") +
			 "\" height=\"" + rs.getInt("height") + "\">\n");
		/*        fw.write("<div class=\"codeBox\"><applet archive=\"" + rs.getString("archive") +
             "\" code=\"applications.exerciseapplet.ExerciseApplet.class" +
             "\" width=\"" + rs.getInt("width") +
             "\" height=\"" + rs.getInt("height") + "\">\n"); */
        Enumeration par = params.keys();
		while (par.hasMoreElements()) {
		    String name = par.nextElement().toString();
		    String value = params.get(name).toString();
		    fw.write("<param name=\"" + name + "\" value=\"" +
			     value + "\"/>\n");
		}
		fw.write("</applet></div>\n");
		
        if(!parser.getTask().equals(""))
            fw.write("</td></tr></table>\n\n");
			 
	    }
	    fw.write("</div></div><br/><hr size=\"1\">\n<div class=\"bottom\"> &nbsp;&nbsp;Created " + new java.util.Date() + " - Powered by <a href=\"http://www.cs.hut.fi/Research/SVG/\">SVG-hut</a></div>\n");
	    fw.write("<br/></body>\n</html>");
	    fw.flush();
        }
    }
        //Create the index page
        
	List sortedNames = removeDuplicates(exerciseNames);
	Collections.sort(sortedNames);
	Hashtable eidToRid = new Hashtable();
	Hashtable rounds = new Hashtable();
	Hashtable roundNames = new Hashtable();
	String rid = null;
	String eidString = null;
	List tempList = null;

	query = "SELECT eid, rid FROM exercises, round WHERE cid=100 AND exercises.rid = round.id;";
	rs = executeQuery(query);
	while(rs.next()) {
	    eidString = rs.getString("eid");
	    rid = rs.getString("rid");
	    eidToRid.put(eidString, rid);
	    if(rounds.get(rid) != null) {
	        tempList = (List)rounds.get(rid);
	        tempList.add(eidString);
	        rounds.put(rid, tempList);
	    }
	    else {
	        tempList = new LinkedList();
	        tempList.add(eidString);
	        rounds.put(rid, tempList);
	    }
	}

	query = "SELECT name, rid FROM round, rounddescription WHERE cid=100 AND lang='en' AND round.id = rounddescription.rid;";        
	rs = executeQuery(query);
	while(rs.next()) {
	    roundNames.put(rs.getString("rid"), rs.getString("name"));
	}

	FileWriter fw = new FileWriter(new File("support/index.html"));

	fw.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n" +
	        "<html>\n<head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n" +
	        "<title>TRAKLA2 - exercise index page </title>" +
	        "<link rev=\"made\" href=\"mailto:matrix@cs.hut.fi\">" +
	        "<link rel=\"stylesheet\" href=\"trakla2.css\" type=\"text/css\">" +
	"</head>\n<body>");
	fw.write("<table width=\"100%\" class=\"borderTable\">" +
	        "<tr><td class=\"borderlink\"><h1 class=\"applicationName\">TRAKLA2</h1></td>" +
	        "<td class=\"borderlink\"><a href=\"index.html\">Index</a> &nbsp;</td>" +
	        "<td class=\"borderlink\"><a href=\"http://www.cs.hut.fi/Research/TRAKLA2/\">Research</a> &nbsp;</td>" +
	        "<td class=\"borderlink\"><a href=\"http://www.cs.hut.fi/Research/TRAKLA2/support.shtml\">Support</a></td>" +
	"</tr></table>\n");
	fw.write("<div class=\"cexerpage\">" +
	"<h2>Exercise index page</h2><div class=\"mainTable\"><div class=\"codeBox\">\n");

	List roundList = new LinkedList(rounds.keySet());
	List exerciseList = new LinkedList();
    Collections.sort(roundList);
    
	for(int i = 0; i < roundList.size(); i++) {
	    fw.write("<h3>" + roundNames.get(roundList.get(i)) + "</h3><ul>");
	    exerciseList = (List)rounds.get(roundList.get(i));
	    for(int j = 0; j < exerciseList.size(); j++) {
	        System.out.println("eid: " + exerciseList.get(j));
            if(eidToExerciseClass.get(exerciseList.get(j)) != null)
                fw.write("<li><a href=\"" + eidToExerciseClass.get(exerciseList.get(j)) + ".html\">" + names.get(new Integer(((String)exerciseList.get(j)))) + "</a></li>");
	    }
        fw.write("</ul><br/>");
	}

	//for(int i = 0;i<sortedNames.size();i++)
	//    fw.write("<a href=\"" + sortedNames.get(i) + ".html\">" + exerciseFullNames.get(sortedNames.get(i)) + "</a><br/>");

	fw.write("</div></div><div class=\"bottom\">Created " + new java.util.Date() + " - Powered by <a href=\"http://www.cs.hut.fi/Research/SVG/\">SVG-hut</a></div>\n");
	fw.write("</div></body>\n</html>");
	fw.flush();


    }

    private static List removeDuplicates(List items) { 
        Set set = new LinkedHashSet();
        set.addAll(items);
        return new ArrayList(set);
    }
}
