package junit.cexer;

import junit.framework.TestCase;
import cexer.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Date;

import org.junit.*;
import static org.junit.Assert.*;
import junit.framework.JUnit4TestAdapter;

public class StudentTest {
    
    private Student student;
    private String studentNumber;
    private String name;
    private Course course, wrongCourse;
    private Round round, round2;
    private Exercise exercise, exercise2;
        
    @Before 
    public void setUp() throws Exception {
        name = "Teemu Teekkari";
        studentNumber = "12345A";
        Student student = new Student(studentNumber, name);
        
        course = new Course("123", "JUnit Test Course");
        wrongCourse = new Course("456", "JUnit Test Course 2. No students.");
        
        round = new Round();
        round.setRoundNumber(1);
        round.setId(101);
        round2 = new Round();
        round2.setRoundNumber(2);
        round.setId(102);
        
        course.addRound(round);
        course.addRound(round2);
        course.setId(1);
        wrongCourse.addRound(round);
        wrongCourse.setId(2);
        
        int maxPoints = 5;
        int exerciseNumber = 2;
        String[] params = new String[8];
        params[0] = "Exercise Name";
        params[1] = "Exercise description";
        params[2] = "Exercise codebase";
        params[3] = "Archive";
        params[4] = "Code";
        params[5] = "100";
        params[6] = "100";        
        params[7] = "Hintpage adress";   
        exercise = new Exercise(new ExerciseTemplate(params,new ArrayList()), maxPoints, exerciseNumber);        
        exercise2 = new Exercise(new ExerciseTemplate(params,new ArrayList()), maxPoints, exerciseNumber + 1);
        round.addExercise(exercise);
        round.addExercise(exercise2);
        student.addCourse(course);
        
    }

    /*
     * Test method for 'cexer.Student.Student(String)'
     */
    @Test
    public void studentString() {
        Student stu = new Student(studentNumber);
        assertEquals(stu.getStudentNumber(), studentNumber);
        assertEquals(stu.getName(), null);
    }

    /*
     * Test method for 'cexer.Student.Student(String, String)'
     */
    @Test
    public void studentStringString() {
        Student stu = new Student(studentNumber, name);
        assertEquals(stu.getStudentNumber(), studentNumber);
        assertEquals(stu.getName(), name);
    }

    /*
     * Test method for 'cexer.Student.setResult(Course, int, int, int)'
     */
    @Test
    public void setResult() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
               
        stu.setResult(course, 1, 1, 2);
        assertEquals(stu.getResults(course).getResult(1,1), 2);
        stu.setResult(course, 0, 1, 3); 
        assertFalse(stu.getResults(course).getResult(-1,0) == 3);
        stu.setResult(course, 1, 0, 4);
        assertFalse(stu.getResults(course).getResult(0,-1) == 4);
        
        assertTrue(stu.getResults(course).getResult(1,2) == 0);
        stu.setResult(course, 1, 2, 1);
        stu.setResult(course, 1, 2, 2);        
        stu.setResult(course, 1, 2, -1); 
        assertFalse(stu.getResults(course).getResult(1,2) == -1);
        assertTrue(stu.getResults(course).getResult(1,2) == 2);
        
        stu.setResult(wrongCourse, 1, 2, 1);
        assertEquals(stu.getResults(wrongCourse), null);
    }

    /*
     * Test method for 'cexer.Student.editResult(Course, Round, Exercise, int)'
     */
    @Test
    public void editResult() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
                
        stu.setResult(course, 1, 2, 2);
        assertTrue(stu.editResult(course, round, exercise, 1));
        assertTrue(stu.getResults(course).getResult(1,2) == 1);
        
        assertFalse(stu.editResult(course, round, exercise, -1));
        assertFalse(stu.editResult(wrongCourse, round, exercise, 1));
        
        assertFalse(stu.editResult(course, null, null, 1));
        assertFalse(stu.editResult(course, round, null, 1));
        assertFalse(stu.editResult(null, round, exercise, 1));
    }

    /*
     * Test method for 'cexer.Student.getResults(Course)'
     */
    @Test
    public void getResultsCourse() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        
        assertTrue(stu.getResults(course) != null);
        assertEquals(stu.getResults(wrongCourse), null);
        
        stu.setInactive(course);
        assertEquals(stu.getResults(course), null);
    }

    /*
     * Test method for 'cexer.Student.getResult(Course, Round, Exercise)'
     */
    @Test
    public void getResult() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);

        assertEquals(stu.getResult(course, round, exercise), 0); 
        stu.setResult(course, 1, 2, 2);
        stu.setResult(course, 1, 3, 1);
        
        assertEquals(stu.getResult(course, round, exercise), 2);         
        assertEquals(stu.getResult(wrongCourse, round, exercise), 0);        

    }

    /*
     * Test method for 'cexer.Student.getSubmissions(Course, Round, Exercise)'
     */
    @Test
    public void getSubmissionsCourseRoundExercise() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        
        assertEquals(stu.getSubmissions(course, round, exercise), 0); 
        
        stu.setResult(course, 1, 2, 2);
        stu.setResult(course, 1, 2, 0);
        stu.setResult(course, 1, 2, 1);
        stu.setResult(course, 1, 3, 0);

        assertEquals(stu.getSubmissions(course, round, exercise), 3);
        assertEquals(stu.getSubmissions(wrongCourse, round, exercise), 0);
        
        assertEquals(stu.getSubmissions(null, round, exercise), 0);
    }

    @Test(expected=NullPointerException.class)
    public void nullPointerExceptionGetSubmissionsCourseRoundExercise() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        
        stu.getSubmissions(course, null, exercise);
        //stu.getSubmissions(course, round, null);
    }
    
    /*
     * Test method for 'cexer.Student.getSubmissions(Course, Round)'
     */
    @Test
    public void getSubmissionsCourseRound() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        
        assertEquals(stu.getSubmissions(course, round), 0); 
        
        stu.setResult(course, 1, 2, 2);
        stu.setResult(course, 1, 2, 0);
        stu.setResult(course, 1, 2, 1);
        stu.setResult(course, 1, 3, 0);

        assertEquals(stu.getSubmissions(course, round), 4);
        assertEquals(stu.getSubmissions(wrongCourse, round), 0);
        
        /* NULL values not allowed, these should cause an error
        assertEquals(stu.getSubmissions(null, round), 0);
        assertEquals(stu.getSubmissions(course, null), 0);
        */
    }

    /*
     * Test method for 'cexer.Student.getRoundResult(Course, int)'
     */
    @Test
    public void getRoundResult() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        stu.setResult(course, 1, 2, 2);
        stu.setResult(course, 1, 3, 0);
        stu.setResult(course, 1, 4, 1);
        stu.setResult(course, 1, 5, 3);       
        
        assertEquals(stu.getRoundResult(course, 1), 6);
        assertEquals(stu.getRoundResult(course, -1), 0);
        assertEquals(stu.getRoundResult(course, 3), 0);
        assertEquals(stu.getRoundResult(wrongCourse, 3), 0);
        assertEquals(stu.getRoundResult(null, 2), 0);
    }

    /*
     * Test method for 'cexer.Student.getCourseResult(Course)'
     */
    @Test
    public void getCourseResult() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        
        assertEquals(stu.getCourseResult(course), 0);
        
        stu.setResult(course, 1, 2, 2);
        stu.setResult(course, 1, 3, 0);
        stu.setResult(course, 1, 4, 1);
        stu.setResult(course, 1, 5, 3);  
        stu.setResult(course, 2, 4, 0);
        stu.setResult(course, 2, 5, 3);   
        
        assertEquals(stu.getCourseResult(course), 9);        
        
        stu.setResult(course, 2, 4, -1);
        assertEquals(stu.getCourseResult(course), 9);
        
        stu.editResult(course, round, exercise, 1);
        assertEquals(stu.getCourseResult(course), 8);

        assertEquals(stu.getCourseResult(wrongCourse), 0);
        assertEquals(stu.getCourseResult(null), 0);
    }

    /*
     * Test method for 'cexer.Student.getGrade(Course)'
     */
    @Test
    public void getGrade() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        
        int[] grades = new int[] {2,6,12,22,32};
        String[] names = {"1", "2", "3", "4", "5"};
        
        course.setGrades(grades, names);
        
        assertEquals(stu.getGrade(course), "0");        
        stu.setResult(course, 1, 2, 2);
        assertEquals(stu.getGrade(course), "1");
        stu.setResult(course, 1, 3, 2);
        stu.setResult(course, 1, 4, 2);
        stu.setResult(course, 1, 5, 4);  
        stu.setResult(course, 2, 4, 0);
        stu.setResult(course, 2, 5, 3);
        assertEquals(stu.getGrade(course), "3");
        
        assertEquals(stu.getGrade(wrongCourse), "0");        
    }

    @Test(expected=NullPointerException.class)
    public void nullPointerExceptionGetGrade() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        assertEquals(stu.getGrade(null), null);
    }
    /*
     * Test method for 'cexer.Student.addDeadLine(int, Date)'
     */
    @Test
    public void addDeadLine() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        
        Date deadline = new Date();
        
        stu.addDeadLine(round.getId(), deadline);
        assertEquals(stu.getDeadLine(round.getId()), deadline);                
        
    }

    /*
     * Test method for 'cexer.Student.getFormattedDeadLine(int)'
     */
    public void testGetFormattedDeadLine() {
        // TODO Auto-generated method stub

    }

    /*
     * Test method for 'cexer.Student.hasPersonalDeadLine(int)'
     */
    @Test
    public void hasPersonalDeadLine() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        
        Date deadline = new Date();
        
        stu.addDeadLine(round.getId(), deadline);
        assertTrue(stu.hasPersonalDeadLine(round.getId()));
        assertFalse(stu.hasPersonalDeadLine(round2.getId()));

    }


    /*
     * Test method for 'cexer.Student.equals(Object)'
     */
    @Test
    public void equalsObject() {
        Student stu = new Student(studentNumber, name);
        Student stu2 = new Student(studentNumber, name);
        Student stu3 = new Student("testNumber", name);
        
        assertTrue(stu.equals(stu2));
        assertFalse(stu.equals(stu3));
        
        stu2.setAccessLevel(2);
        assertFalse(stu.equals(stu2));       
        assertFalse(stu.equals(new String()));               
    }


    /*
     * Test method for 'cexer.Student.setInactive(Course)'
     */
    @Test
    public void setInactive() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        assertTrue(stu.isActive(course));
        stu.setInactive(course);
        assertFalse(stu.isActive(course));
   }

    /*
     * Test method for 'cexer.Student.setActive(Course)'
     */
    @Test
    public void setActive() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        stu.setInactive(course);
        stu.setActive(course);
        assertTrue(stu.isActive(course)); 
    }

    /*
     * Test method for 'cexer.Student.getAllCourses()'
     */
    @Test
    public void getAllCourses() {
        Student stu = new Student(studentNumber, name);
        stu.addCourse(course);
        stu.addCourse(wrongCourse);
        assertEquals(stu.getAllCourses().size(), 2);
        
        stu.addCourse(course);
        assertEquals(stu.getAllCourses().size(), 2);
        
        stu.setInactive(wrongCourse);
        assertEquals(stu.getAllCourses().size(), 2);        

    }

    /*
     * Test method for 'cexer.Student.compareTo(Object)'
     */
    @Test
    public void testCompareTo() {
        Student stu = new Student(studentNumber, "Aa");
        Student stu2 = new Student(studentNumber, "Aa");
        Student stu3 = new Student(studentNumber, "ab");
        Student stu4 = new Student(studentNumber, "Ca");
        
        assertEquals(stu.compareTo(stu2), 0);
        assertTrue(stu2.compareTo(stu3) < 0);
        assertTrue(stu4.compareTo(stu3) > 0);

    }
    
    @Test(expected=ClassCastException.class)
    public void classCastCompareTo() {
        Student stu = new Student(studentNumber, name);
        stu.compareTo(course);
    }
    
    public static junit.framework.Test suite() {
        return new JUnit4TestAdapter(StudentTest.class);
    }

}
